#include <math.h>
#include "Faces.h"

kface CalculateNormals(kface theFace)
{
  double v1[3], v2[3], l;
  v1[0] = theFace.VArray[0].X - theFace.VArray[1].X;
  v1[1] = theFace.VArray[0].Y - theFace.VArray[1].Y;
  v1[2] = theFace.VArray[0].Z - theFace.VArray[1].Z;
  v2[0] = theFace.VArray[2].X - theFace.VArray[1].X;
  v2[1] = theFace.VArray[2].Y - theFace.VArray[1].Y;
  v2[2] = theFace.VArray[2].Z - theFace.VArray[1].Z;
                                            
  theFace.NormalX = v1[1]*v2[2] - v2[1]*v1[2];
  theFace.NormalY = v1[2]*v2[0] - v2[2]*v1[0];
  theFace.NormalZ = v1[0]*v2[1] - v2[0]*v1[1];

  l = sqrt(theFace.NormalX*theFace.NormalX + theFace.NormalY*theFace.NormalY + 
           theFace.NormalZ*theFace.NormalZ);
  theFace.NormalX /= l;
  theFace.NormalY /= l;
  theFace.NormalZ /= l;
  
  return theFace;            
}

kvertex CreateVertex(double x, double y, double z)
{
 kvertex newVertex;
 newVertex.X = x;
 newVertex.Y = y;
 newVertex.Z = z;  

 return newVertex;       
}

void freeList(klist theList)
{
	theList.Current = theList.Head;

	while (theList.Current != NULL) 
	{ 	 
		theList.Current = theList.Head->next;
		free(theList.Head);
		theList.Head = theList.Current;
	}     
}
klist addNode(klist theList, kface theFace)
{
	knode* newNode = (knode*)malloc(sizeof(knode));
 
	newNode->theFace = theFace;
	newNode->next = NULL;
 	
	if (theList.Head == NULL)
	{
		theList.Head = newNode;
		theList.Current = theList.Head;
		theList.Head->next = NULL;
		theList.Current->next = NULL;
	}
	else
	{
		theList.Current = theList.Head;  
		while(theList.Current->next != NULL)
		{
			theList.Current = theList.Current->next;
        }

		theList.Current->next = newNode;
	}

	return theList;            
}

klist ResetList(klist theList)
{
  	theList.Current = theList.Head;
	return theList;         
}

klist CreateEmptyList()
{
 	klist theList;
	theList.Head = NULL;
	theList.Current = NULL;
	theList.Current = theList.Head;
 
	return theList;     
}

klist CreateConnector(klist theList, kvertex BL1, kvertex BR1, kvertex TL1, kvertex TR1,
                      kvertex BL2, kvertex BR2, kvertex TL2, kvertex TR2)
{
 kface Side1;
 Side1.NumVertices = 4;
 Side1.VArray[0] = BL1;
 Side1.VArray[1] = BR1;
 Side1.VArray[2] = BR2;
 Side1.VArray[3] = BL2;
 Side1 = CalculateNormals(Side1);
 theList = addNode(theList, Side1);   

 kface Side2;
 Side2.NumVertices = 4;
 Side2.VArray[0] = BL1;
 Side2.VArray[1] = TL1;
 Side2.VArray[2] = TL2;
 Side2.VArray[3] = BL2;
 Side2 = CalculateNormals(Side2);
 theList = addNode(theList, Side2);    

 kface Side3;
 Side3.NumVertices = 4;
 Side3.VArray[0] = BR1;
 Side3.VArray[1] = TR1;
 Side3.VArray[2] = TR2;
 Side3.VArray[3] = BR2;
 Side3 = CalculateNormals(Side3);
 theList = addNode(theList, Side3);     

 kface Side4;
 Side4.NumVertices = 4;
 Side4.VArray[0] = TL1;
 Side4.VArray[1] = TR1;
 Side4.VArray[2] = TR2;
 Side4.VArray[3] = TL2;
 Side4 = CalculateNormals(Side4);
 theList = addNode(theList, Side4);      
 
 return theList;
} 
klist CreateBoxRim()
{
  klist newList = CreateEmptyList();
  kvertex BL1, BL2, BR1, BR2, TL1, TL2, TR1, TR2;
  kface Side1;
  
  BL1 = CreateVertex(0.05, 0.0, 0.05);
  BR1 = CreateVertex(0.05, 0.0, 0.0);
  TL1 = CreateVertex(0.05, 0.05, 0.05);  
  TR1 = CreateVertex(0.05, 0.05, 0.0);
  BL2 = CreateVertex(0.95, 0.0, 0.05);
  BR2 = CreateVertex(0.95, 0.0, 0.0);
  TL2 = CreateVertex(0.95, 0.05, 0.05);  
  TR2 = CreateVertex(0.95, 0.05, 0.0);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(0.05, 1.0, 0.05);
  BR1 = CreateVertex(0.05, 1.0, 0.0);
  TL1 = CreateVertex(0.05, 0.95, 0.05);  
  TR1 = CreateVertex(0.05, 0.95, 0.0);
  BL2 = CreateVertex(0.95, 1.0, 0.05);
  BR2 = CreateVertex(0.95, 1.0, 0.0);
  TL2 = CreateVertex(0.95, 0.95, 0.05);  
  TR2 = CreateVertex(0.95, 0.95, 0.0);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(0.05, 0.0, 0.95);
  BR1 = CreateVertex(0.05, 0.0, 1.0);
  TL1 = CreateVertex(0.05, 0.05, 0.95);  
  TR1 = CreateVertex(0.05, 0.05, 1.0);
  BL2 = CreateVertex(0.95, 0.0, 0.95);
  BR2 = CreateVertex(0.95, 0.0, 1.0);
  TL2 = CreateVertex(0.95, 0.05, 0.95);  
  TR2 = CreateVertex(0.95, 0.05, 1.0);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(0.05, 1.0, 0.95);
  BR1 = CreateVertex(0.05, 1.0, 1.0);
  TL1 = CreateVertex(0.05, 0.95, 0.95);  
  TR1 = CreateVertex(0.05, 0.95, 1.0);
  BL2 = CreateVertex(0.95, 1.0, 0.95);
  BR2 = CreateVertex(0.95, 1.0, 1.0);
  TL2 = CreateVertex(0.95, 0.95, 0.95);  
  TR2 = CreateVertex(0.95, 0.95, 1.0);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(0.0, 0.0, 0.05);
  BR1 = CreateVertex(0.05, 0.0, 0.05);
  TL1 = CreateVertex(0.0, 0.05, 0.05);  
  TR1 = CreateVertex(0.05, 0.05, 0.05);
  BL2 = CreateVertex(0.0, 0.0, 0.95);
  BR2 = CreateVertex(0.05, 0.0, 0.95);
  TL2 = CreateVertex(0.0, 0.05, 0.95);  
  TR2 = CreateVertex(0.05, 0.05, 0.95);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(0.0, 1.0, 0.05);
  BR1 = CreateVertex(0.05, 1.0, 0.05);
  TL1 = CreateVertex(0.0, 0.95, 0.05);  
  TR1 = CreateVertex(0.05, 0.95, 0.05);
  BL2 = CreateVertex(0.0, 1.0, 0.95);
  BR2 = CreateVertex(0.05, 1.0, 0.95);
  TL2 = CreateVertex(0.0, 0.95, 0.95);  
  TR2 = CreateVertex(0.05, 0.95, 0.95);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(1.0, 0.0, 0.05);
  BR1 = CreateVertex(0.95, 0.0, 0.05);
  TL1 = CreateVertex(1.0, 0.05, 0.05);  
  TR1 = CreateVertex(0.95, 0.05, 0.05);
  BL2 = CreateVertex(1.0, 0.0, 0.95);
  BR2 = CreateVertex(0.95, 0.0, 0.95);
  TL2 = CreateVertex(1.0, 0.05, 0.95);  
  TR2 = CreateVertex(0.95, 0.05, 0.95);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);

  BL1 = CreateVertex(1.0, 1.0, 0.05);
  BR1 = CreateVertex(0.95, 1.0, 0.05);
  TL1 = CreateVertex(1.0, 0.95, 0.05);  
  TR1 = CreateVertex(0.95, 0.95, 0.05);
  BL2 = CreateVertex(1.0, 1.0, 0.95);
  BR2 = CreateVertex(0.95, 1.0, 0.95);
  TL2 = CreateVertex(1.0, 0.95, 0.95);  
  TR2 = CreateVertex(0.95, 0.95, 0.95);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);  
  
  BL1 = CreateVertex(0.0, 0.0, 0.0);
  BR1 = CreateVertex(0.05, 0.0, 0.0);
  TL1 = CreateVertex(0.0, 0.0, 0.05);  
  TR1 = CreateVertex(0.05, 0.0, 0.05);
  BL2 = CreateVertex(0.0, 1.0, 0.0);
  BR2 = CreateVertex(0.05, 1.0, 0.0);
  TL2 = CreateVertex(0.0, 1.0, 0.05);  
  TR2 = CreateVertex(0.05, 1.0, 0.05);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);   
  Side1.NumVertices = 4;
  Side1.VArray[0] = BL1;
  Side1.VArray[1] = BR1;
  Side1.VArray[2] = TR1;
  Side1.VArray[3] = TL1;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);  
  Side1.VArray[0] = BL2;
  Side1.VArray[1] = BR2;
  Side1.VArray[2] = TR2;
  Side1.VArray[3] = TL2;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);

  BL1 = CreateVertex(1.0, 1.0, 1.0);
  BR1 = CreateVertex(0.95, 1.0, 1.0);
  TL1 = CreateVertex(1.0, 1.0, 0.95);  
  TR1 = CreateVertex(0.95, 1.0, 0.95);
  BL2 = CreateVertex(1.0, 0.0, 1.0);
  BR2 = CreateVertex(0.95, 0.0, 1.0);
  TL2 = CreateVertex(1.0, 0.0, 0.95);  
  TR2 = CreateVertex(0.95, 0.0, 0.95);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);   
  Side1.NumVertices = 4;
  Side1.VArray[0] = BL1;
  Side1.VArray[1] = BR1;
  Side1.VArray[2] = TR1;
  Side1.VArray[3] = TL1;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);  
  Side1.VArray[0] = BL2;
  Side1.VArray[1] = BR2;
  Side1.VArray[2] = TR2;
  Side1.VArray[3] = TL2;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);  

  BL1 = CreateVertex(1.0, 1.0, 0.0);
  BR1 = CreateVertex(0.95, 1.0, 0.0);
  TL1 = CreateVertex(1.0, 1.0, 0.05);  
  TR1 = CreateVertex(0.95, 1.0, 0.05);
  BL2 = CreateVertex(1.0, 0.0, 0.0);
  BR2 = CreateVertex(0.95, 0.0, 0.0);
  TL2 = CreateVertex(1.0, 0.0, 0.05);  
  TR2 = CreateVertex(0.95, 0.0, 0.05);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);   
  Side1.NumVertices = 4;
  Side1.VArray[0] = BL1;
  Side1.VArray[1] = BR1;
  Side1.VArray[2] = TR1;
  Side1.VArray[3] = TL1;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);  
  Side1.VArray[0] = BL2;
  Side1.VArray[1] = BR2;
  Side1.VArray[2] = TR2;
  Side1.VArray[3] = TL2;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);  

  BL1 = CreateVertex(0.0, 1.0, 1.0);
  BR1 = CreateVertex(0.05, 1.0, 1.0);
  TL1 = CreateVertex(0.0, 1.0, 0.95);  
  TR1 = CreateVertex(0.05, 1.0, 0.95);
  BL2 = CreateVertex(0.0, 0.0, 1.0);
  BR2 = CreateVertex(0.05, 0.0, 1.0);
  TL2 = CreateVertex(0.0, 0.0, 0.95);  
  TR2 = CreateVertex(0.05, 0.0, 0.95);
  newList = CreateConnector(newList, BL1, BR1, TL1, TR1, BL2, BR2, TL2, TR2);   
  Side1.NumVertices = 4;
  Side1.VArray[0] = BL1;
  Side1.VArray[1] = BR1;
  Side1.VArray[2] = TR1;
  Side1.VArray[3] = TL1;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);  
  Side1.VArray[0] = BL2;
  Side1.VArray[1] = BR2;
  Side1.VArray[2] = TR2;
  Side1.VArray[3] = TL2;
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);    
    
  return newList;
}

klist CreateCrystal()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 3;
 for(int i = 0; i < 8; i++)
 {
  Side.VArray[0] = CreateVertex(0.5, 1.0, 0.5);  
  if(i == 0) Side.VArray[1] = CreateVertex(0.5, 0.5, 1.0);
  else Side.VArray[1] = Side.VArray[2];
  if(i == 0) Side.VArray[2] = CreateVertex(0.15, 0.5, 0.85);
  else if(i == 1) Side.VArray[2] = CreateVertex(0.0, 0.5, 0.5);
  else if(i == 2) Side.VArray[2] = CreateVertex(0.15, 0.5, 0.15);
  else if(i == 3) Side.VArray[2] = CreateVertex(0.5, 0.5, 0.0);
  else if(i == 4) Side.VArray[2] = CreateVertex(0.85, 0.5, 0.15);
  else if(i == 5) Side.VArray[2] = CreateVertex(1.0, 0.5, 0.5);
  else if(i == 6) Side.VArray[2] = CreateVertex(0.85, 0.5, 0.85);
  else if(i == 7) Side.VArray[2] = CreateVertex(0.5, 0.5, 1.0);
  Side = CalculateNormals(Side);
  newList = addNode(newList, Side);  
  Side.VArray[0] = CreateVertex(0.5, 0.0, 0.5);
  Side = CalculateNormals(Side);
  newList = addNode(newList, Side);
 }          
 return newList;
}

klist CreateArrowStem()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;     
 Side.VArray[0] = CreateVertex(0.4,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.0, 0.5);
 Side.VArray[3] = CreateVertex(0.4, 0.0, 0.5); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.4,0.1,0.0);
 Side.VArray[1] = CreateVertex(0.6, 0.1, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.1, 0.5);
 Side.VArray[3] = CreateVertex(0.4, 0.1, 0.5); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 return newList;     
}

klist CreateArrowTop()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 3;     
 Side.VArray[0] = CreateVertex(0.1,0.0,0.5);
 Side.VArray[1] = CreateVertex(0.9, 0.0, 0.5);
 Side.VArray[2] = CreateVertex(0.5, 0.0, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.1,0.1,0.5);
 Side.VArray[1] = CreateVertex(0.9, 0.1, 0.5);
 Side.VArray[2] = CreateVertex(0.5, 0.1, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 return newList;     
}

klist CreateArrowRim()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;     
 Side.VArray[0] = CreateVertex(0.4,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.1, 0.0);
 Side.VArray[3] = CreateVertex(0.4,0.1,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.4,0.1,0.0);
 Side.VArray[1] = CreateVertex(0.4,0.0,0.0);
 Side.VArray[2] = CreateVertex(0.4, 0.0, 0.5); 
 Side.VArray[3] = CreateVertex(0.4, 0.1, 0.5);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.6,0.1,0.0); 
 Side.VArray[1] = CreateVertex(0.6,0.0,0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.0, 0.5); 
 Side.VArray[3] = CreateVertex(0.6, 0.1, 0.5);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.1,0.1,0.5); 
 Side.VArray[1] = CreateVertex(0.1,0.0,0.5);
 Side.VArray[2] = CreateVertex(0.9, 0.0, 0.5); 
 Side.VArray[3] = CreateVertex(0.9, 0.1, 0.5); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.1,0.1,0.5); 
 Side.VArray[1] = CreateVertex(0.1,0.0,0.5);
 Side.VArray[2] = CreateVertex(0.5, 0.0, 1.0); 
 Side.VArray[3] = CreateVertex(0.5, 0.1, 1.0); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.9,0.1,0.5); 
 Side.VArray[1] = CreateVertex(0.9,0.0,0.5);
 Side.VArray[2] = CreateVertex(0.5, 0.0, 1.0); 
 Side.VArray[3] = CreateVertex(0.5, 0.1, 1.0); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     
 
 return newList;           
}

klist CreateExit()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;     
 Side.VArray[0] = CreateVertex(0.5,0.0,0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.8);
 Side.VArray[2] = CreateVertex(1.0, 0.1, 0.8);
 Side.VArray[3] = CreateVertex(0.5,0.1,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);

 Side.VArray[0] = CreateVertex(0.5,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.0, 0.0, 0.8);
 Side.VArray[2] = CreateVertex(0.0, 0.1, 0.8);
 Side.VArray[3] = CreateVertex(0.5,0.1,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.0,0.0,0.8);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.8);
 Side.VArray[2] = CreateVertex(1.0, 0.1, 0.8);
 Side.VArray[3] = CreateVertex(0.0,0.1,0.8);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.5,0.0,1.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.2);
 Side.VArray[2] = CreateVertex(1.0, 0.1, 0.2);
 Side.VArray[3] = CreateVertex(0.5,0.1,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);

 Side.VArray[0] = CreateVertex(0.5,0.0,1.0);
 Side.VArray[1] = CreateVertex(0.0, 0.0, 0.2);
 Side.VArray[2] = CreateVertex(0.0, 0.1, 0.2);
 Side.VArray[3] = CreateVertex(0.5,0.1,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.0,0.0,0.2);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.2);
 Side.VArray[2] = CreateVertex(1.0, 0.1, 0.2);
 Side.VArray[3] = CreateVertex(0.0,0.1,0.2);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 
 return newList;
}

klist CreateTrapEdge()
{
 klist newList = CreateEmptyList();     
 kface Side;
 Side.NumVertices = 4;     
 Side.VArray[0] = CreateVertex(0.5,0.4,0.0);
 Side.VArray[1] = CreateVertex(0.5, 0.6, 0.0);
 Side.VArray[2] = CreateVertex(0.5, 0.6, 1.0);
 Side.VArray[3] = CreateVertex(0.5,0.4,1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.4,0.5,0.0);
 Side.VArray[1] = CreateVertex(0.6, 0.5, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.5, 1.0);
 Side.VArray[3] = CreateVertex(0.4,0.5,1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0,0.4,0.5);
 Side.VArray[1] = CreateVertex(0.0, 0.6, 0.5);
 Side.VArray[2] = CreateVertex(1.0, 0.6, 0.5);
 Side.VArray[3] = CreateVertex(1.0,0.4,0.5);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.4,0.5,0.0);
 Side.VArray[1] = CreateVertex(0.6, 0.5, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.5, 1.0);
 Side.VArray[3] = CreateVertex(0.4,0.5,1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0, 0.5, 0.4);
 Side.VArray[1] = CreateVertex(0.0, 0.5, 0.6);
 Side.VArray[2] = CreateVertex(1.0, 0.5, 0.6);
 Side.VArray[3] = CreateVertex(1.0, 0.5, 0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.4, 0.0, 0.5);
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.5);
 Side.VArray[2] = CreateVertex(0.6, 1.0, 0.5);
 Side.VArray[3] = CreateVertex(0.4, 1.0, 0.5);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.5, 0.0, 0.4);
 Side.VArray[1] = CreateVertex(0.5, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.5, 1.0, 0.6);
 Side.VArray[3] = CreateVertex(0.5, 1.0, 0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    
 
 return newList; 
}

klist CreateSwitchBox()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;     
 Side.VArray[0] = CreateVertex(0.0,0.0,0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.0, 1.0);
 Side.VArray[3] = CreateVertex(0.0,0.0,1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.0,0.2,0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.2, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.2, 1.0);
 Side.VArray[3] = CreateVertex(0.0,0.2,1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0,0.0,0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.2, 0.0);
 Side.VArray[3] = CreateVertex(0.0,0.2,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.0,0.0,1.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 1.0);
 Side.VArray[2] = CreateVertex(1.0, 0.2, 1.0);
 Side.VArray[3] = CreateVertex(0.0,0.2,1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.0,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.0, 0.0, 1.0);
 Side.VArray[2] = CreateVertex(0.0, 0.2, 1.0);
 Side.VArray[3] = CreateVertex(0.0,0.2,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(1.0,0.0,0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 1.0);
 Side.VArray[2] = CreateVertex(1.0, 0.2, 1.0);
 Side.VArray[3] = CreateVertex(1.0,0.2,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      
     
 return newList;
}

klist CreatePidFace()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;
 
 Side.VArray[0] = CreateVertex(0.3,0.6,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.6, 0.3);
 Side.VArray[2] = CreateVertex(0.7, 0.9, 0.3);
 Side.VArray[3] = CreateVertex(0.3,0.9,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   
   
 return newList;
}

klist CreatePidFront()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;      
 Side.VArray[0] = CreateVertex(0.25,0.1,0.25);
 Side.VArray[1] = CreateVertex(0.75, 0.1, 0.25);
 Side.VArray[2] = CreateVertex(0.75, 0.6, 0.25);
 Side.VArray[3] = CreateVertex(0.25,0.6,0.25);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.25,0.1,0.25);
 Side.VArray[1] = CreateVertex(0.75, 0.1, 0.25);
 Side.VArray[2] = CreateVertex(0.75, 0.1, 0.75);
 Side.VArray[3] = CreateVertex(0.25,0.1,0.75);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 return newList;      
}

klist CreatePidFeet()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.15,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.45, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(0.4, 0.0, 0.6);
 Side.VArray[3] = CreateVertex(0.25,0.0,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.15,0.05,0.0);
 Side.VArray[1] = CreateVertex(0.45, 0.05, 0.0);
 Side.VArray[2] = CreateVertex(0.4, 0.1, 0.6);
 Side.VArray[3] = CreateVertex(0.25,0.1,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.25,0.0,0.6);
 Side.VArray[1] = CreateVertex(0.4, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.4, 0.1, 0.6);
 Side.VArray[3] = CreateVertex(0.25,0.1,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.25,0.0,0.6);
 Side.VArray[1] = CreateVertex(0.25, 0.1, 0.6);
 Side.VArray[2] = CreateVertex(0.15, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.15,0.0,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(0.4,0.0,0.6);
 Side.VArray[1] = CreateVertex(0.4, 0.1, 0.6);
 Side.VArray[2] = CreateVertex(0.45, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.45,0.0,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.15,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.15, 0.05, 0.0);
 Side.VArray[2] = CreateVertex(0.45, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.45,0.0,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   
//left foot
 Side.VArray[0] = CreateVertex(0.85,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.55, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.0, 0.6);
 Side.VArray[3] = CreateVertex(0.75,0.0,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.85,0.05,0.0);
 Side.VArray[1] = CreateVertex(0.55, 0.05, 0.0);
 Side.VArray[2] = CreateVertex(0.6, 0.1, 0.6);
 Side.VArray[3] = CreateVertex(0.75,0.1,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.75,0.0,0.6);
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.6, 0.1, 0.6);
 Side.VArray[3] = CreateVertex(0.75,0.1,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.75,0.0,0.6);
 Side.VArray[1] = CreateVertex(0.75, 0.1, 0.6);
 Side.VArray[2] = CreateVertex(0.85, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.85,0.0,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(0.6,0.0,0.6);
 Side.VArray[1] = CreateVertex(0.6, 0.1, 0.6);
 Side.VArray[2] = CreateVertex(0.55, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.55,0.0,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.85,0.0,0.0);
 Side.VArray[1] = CreateVertex(0.85, 0.05, 0.0);
 Side.VArray[2] = CreateVertex(0.55, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.55,0.0,0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   
//beak
 Side.VArray[0] = CreateVertex(0.5,0.7,0.3);
 Side.VArray[1] = CreateVertex(0.6, 0.65, 0.3);
 Side.VArray[2] = CreateVertex(0.5, 0.65, 0.2);
 Side.VArray[3] = CreateVertex(0.5,0.65,0.2);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.5,0.7,0.3);
 Side.VArray[1] = CreateVertex(0.4, 0.65, 0.3);
 Side.VArray[2] = CreateVertex(0.5, 0.65, 0.2);
 Side.VArray[3] = CreateVertex(0.5,0.65,0.2);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.4,0.65,0.3);
 Side.VArray[1] = CreateVertex(0.6, 0.65, 0.3);
 Side.VArray[2] = CreateVertex(0.5, 0.65, 0.2);
 Side.VArray[3] = CreateVertex(0.5,0.65,0.2);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 return newList;
}

klist CreateGolemEnds()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.0, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.0, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0, 0.05, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.05, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.05, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.05, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.05, 0.0);
 Side.VArray[3] = CreateVertex(0.0, 0.05, 0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.0, 0.0, 1.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 1.0);
 Side.VArray[2] = CreateVertex(1.0, 0.05, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.05, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(1.0, 0.0, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.0, 1.0);
 Side.VArray[2] = CreateVertex(1.0, 0.05, 1.0);
 Side.VArray[3] = CreateVertex(1.0, 0.05, 0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0);
 Side.VArray[1] = CreateVertex(0.0, 0.0, 1.0);
 Side.VArray[2] = CreateVertex(0.0, 0.05, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.05, 0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       
 //bottom
 Side.VArray[0] = CreateVertex(0.0, 1.0, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 1.0, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 1.0, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 1.0, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0, 0.95, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 0.95, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.95, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.95, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.0, 1.0, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 1.0, 0.0);
 Side.VArray[2] = CreateVertex(1.0, 0.95, 0.0);
 Side.VArray[3] = CreateVertex(0.0, 0.95, 0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.0, 1.0, 1.0);
 Side.VArray[1] = CreateVertex(1.0, 1.0, 1.0);
 Side.VArray[2] = CreateVertex(1.0, 0.95, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.95, 1.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(1.0, 1.0, 0.0);
 Side.VArray[1] = CreateVertex(1.0, 1.0, 1.0);
 Side.VArray[2] = CreateVertex(1.0, 0.95, 1.0);
 Side.VArray[3] = CreateVertex(1.0, 0.95, 0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.0, 1.0, 0.0);
 Side.VArray[1] = CreateVertex(0.0, 1.0, 1.0);
 Side.VArray[2] = CreateVertex(0.0, 0.95, 1.0);
 Side.VArray[3] = CreateVertex(0.0, 0.95, 0.0);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    
 return newList;      
}

klist CreatePidArms(klist newList)
{
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.25,0.6,0.4);
 Side.VArray[1] = CreateVertex(0.25, 0.6, 0.6);
 Side.VArray[2] = CreateVertex(0.1, 0.2, 0.6);
 Side.VArray[3] = CreateVertex(0.1,0.2,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.25,0.4,0.3);
 Side.VArray[1] = CreateVertex(0.25, 0.4, 0.7);
 Side.VArray[2] = CreateVertex(0.15, 0.2, 0.7);
 Side.VArray[3] = CreateVertex(0.15,0.2,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.25,0.4,0.3);
 Side.VArray[1] = CreateVertex(0.25, 0.6, 0.4);
 Side.VArray[2] = CreateVertex(0.1, 0.2, 0.4);
 Side.VArray[3] = CreateVertex(0.15,0.2,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.25,0.6,0.6);
 Side.VArray[1] = CreateVertex(0.25, 0.4, 0.7);
 Side.VArray[2] = CreateVertex(0.15, 0.2, 0.7);
 Side.VArray[3] = CreateVertex(0.1,0.2,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.1, 0.2, 0.6);
 Side.VArray[1] = CreateVertex(0.1,0.2,0.4);
 Side.VArray[2] = CreateVertex(0.15, 0.2, 0.7);
 Side.VArray[3] = CreateVertex(0.15,0.2,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

//Left arm
 Side.VArray[0] = CreateVertex(0.75,0.6,0.4);
 Side.VArray[1] = CreateVertex(0.75, 0.6, 0.6);
 Side.VArray[2] = CreateVertex(0.9, 0.2, 0.6);
 Side.VArray[3] = CreateVertex(0.9,0.2,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.75,0.4,0.3);
 Side.VArray[1] = CreateVertex(0.75, 0.4, 0.7);
 Side.VArray[2] = CreateVertex(0.85, 0.2, 0.7);
 Side.VArray[3] = CreateVertex(0.85,0.2,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.75,0.4,0.3);
 Side.VArray[1] = CreateVertex(0.75, 0.6, 0.4);
 Side.VArray[2] = CreateVertex(0.9, 0.2, 0.4);
 Side.VArray[3] = CreateVertex(0.85,0.2,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.75,0.6,0.6);
 Side.VArray[1] = CreateVertex(0.75, 0.4, 0.7);
 Side.VArray[2] = CreateVertex(0.85, 0.2, 0.7);
 Side.VArray[3] = CreateVertex(0.9,0.2,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.9, 0.2, 0.6);
 Side.VArray[1] = CreateVertex(0.9,0.2,0.4);
 Side.VArray[2] = CreateVertex(0.85, 0.2, 0.7);
 Side.VArray[3] = CreateVertex(0.85,0.2,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     
 
 return newList;
}

klist CreatePidBody()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;
 
 Side.VArray[0] = CreateVertex(0.3,0.6,0.7);
 Side.VArray[1] = CreateVertex(0.7, 0.6, 0.7);
 Side.VArray[2] = CreateVertex(0.7, 0.9, 0.7);
 Side.VArray[3] = CreateVertex(0.3,0.9,0.7);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.3,0.6,0.3);
 Side.VArray[1] = CreateVertex(0.3, 0.6, 0.7);
 Side.VArray[2] = CreateVertex(0.3, 0.9, 0.7);
 Side.VArray[3] = CreateVertex(0.3,0.9,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.7,0.6,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.6, 0.7);
 Side.VArray[2] = CreateVertex(0.7, 0.9, 0.7);
 Side.VArray[3] = CreateVertex(0.7,0.9,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    
 
 Side.VArray[0] = CreateVertex(0.4, 1.0, 0.4);
 Side.VArray[1] = CreateVertex(0.6, 1.0, 0.4);
 Side.VArray[2] = CreateVertex(0.6, 1.0, 0.6);
 Side.VArray[3] = CreateVertex(0.4, 1.0, 0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.3,0.9,0.7);
 Side.VArray[1] = CreateVertex(0.7, 0.9, 0.7);
 Side.VArray[2] = CreateVertex(0.6, 1.0, 0.6);
 Side.VArray[3] = CreateVertex(0.4,1.0,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.3,0.9,0.3);
 Side.VArray[1] = CreateVertex(0.3, 0.9, 0.7);
 Side.VArray[2] = CreateVertex(0.4, 1.0, 0.6);
 Side.VArray[3] = CreateVertex(0.4,1.0,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.7,0.9,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.9, 0.7);
 Side.VArray[2] = CreateVertex(0.6, 1.0, 0.6);
 Side.VArray[3] = CreateVertex(0.6,1.0,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(0.3,0.9,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.9, 0.3);
 Side.VArray[2] = CreateVertex(0.6, 1.0, 0.4);
 Side.VArray[3] = CreateVertex(0.4,1.0,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.25, 0.1, 0.25);
 Side.VArray[1] = CreateVertex(0.25, 0.6, 0.25);
 Side.VArray[2] = CreateVertex(0.25, 0.6, 0.75);
 Side.VArray[3] = CreateVertex(0.25, 0.1, 0.75);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.75, 0.1, 0.25);
 Side.VArray[1] = CreateVertex(0.75, 0.6, 0.25);
 Side.VArray[2] = CreateVertex(0.75, 0.6, 0.75);
 Side.VArray[3] = CreateVertex(0.75, 0.1, 0.75);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.25, 0.6, 0.25);
 Side.VArray[1] = CreateVertex(0.75, 0.6, 0.25);
 Side.VArray[2] = CreateVertex(0.75, 0.6, 0.75);
 Side.VArray[3] = CreateVertex(0.25, 0.6, 0.75);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.25, 0.1, 0.75);
 Side.VArray[1] = CreateVertex(0.25, 0.6, 0.75);
 Side.VArray[2] = CreateVertex(0.75, 0.6, 0.75);
 Side.VArray[3] = CreateVertex(0.75, 0.1, 0.75);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 newList = CreatePidArms(newList);

 return newList;
}

klist CreateSwitch()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 3;
 Side.VArray[0] = CreateVertex(0.4,0.5,0.4);
 Side.VArray[1] = CreateVertex(0.6, 0.5, 0.4);
 Side.VArray[2] = CreateVertex(0.6, 0.5, 0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.6,0.5,0.6);
 Side.VArray[1] = CreateVertex(0.4, 0.5, 0.6);
 Side.VArray[2] = CreateVertex(0.4, 0.5, 0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   
 
 Side.VArray[0] = CreateVertex(0.5, 0.2, 0.05); 
 Side.VArray[1] = CreateVertex(0.6, 0.5, 0.4);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.5, 0.2, 0.05); 
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.4);
 Side.VArray[2] = CreateVertex(0.6,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       

 Side.VArray[0] = CreateVertex(0.5, 0.2, 0.05); 
 Side.VArray[1] = CreateVertex(0.4, 0.0, 0.4);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       

 Side.VArray[0] = CreateVertex(0.5, 0.2, 0.95); 
 Side.VArray[1] = CreateVertex(0.6, 0.5, 0.6);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.5, 0.2, 0.95); 
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.6,0.5,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       

 Side.VArray[0] = CreateVertex(0.5, 0.2, 0.95); 
 Side.VArray[1] = CreateVertex(0.4, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 Side.VArray[0] = CreateVertex(0.05, 0.2, 0.5); 
 Side.VArray[1] = CreateVertex(0.4, 0.5, 0.6);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.05, 0.2, 0.5); 
 Side.VArray[1] = CreateVertex(0.4, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       

 Side.VArray[0] = CreateVertex(0.05, 0.2, 0.5); 
 Side.VArray[1] = CreateVertex(0.4, 0.0, 0.4);
 Side.VArray[2] = CreateVertex(0.4,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       

 Side.VArray[0] = CreateVertex(0.95, 0.2, 0.5); 
 Side.VArray[1] = CreateVertex(0.6, 0.5, 0.6);
 Side.VArray[2] = CreateVertex(0.6,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.95, 0.2, 0.5); 
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.6);
 Side.VArray[2] = CreateVertex(0.6,0.5,0.6);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);       

 Side.VArray[0] = CreateVertex(0.95, 0.2, 0.5); 
 Side.VArray[1] = CreateVertex(0.6, 0.0, 0.4);
 Side.VArray[2] = CreateVertex(0.6,0.5,0.4);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 return newList;       
}

klist CreateTrap()
{
 klist newList = CreateEmptyList();
 
 kface Side;
 Side.NumVertices = 4;     
 Side.VArray[0] = CreateVertex(0.3,0.3,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.3, 0.3);
 Side.VArray[2] = CreateVertex(0.7, 0.3, 0.7);
 Side.VArray[3] = CreateVertex(0.3,0.3,0.7);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.3,0.7,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.7, 0.3);
 Side.VArray[2] = CreateVertex(0.7, 0.7, 0.7);
 Side.VArray[3] = CreateVertex(0.3,0.7,0.7);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.3,0.3,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.3, 0.3);
 Side.VArray[2] = CreateVertex(0.7, 0.7, 0.3);
 Side.VArray[3] = CreateVertex(0.3,0.7,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.3,0.3,0.7);
 Side.VArray[1] = CreateVertex(0.7, 0.3, 0.7);
 Side.VArray[2] = CreateVertex(0.7, 0.7, 0.7);
 Side.VArray[3] = CreateVertex(0.3,0.7,0.7);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.3,0.3,0.3);
 Side.VArray[1] = CreateVertex(0.3, 0.3, 0.7);
 Side.VArray[2] = CreateVertex(0.3, 0.7, 0.7);
 Side.VArray[3] = CreateVertex(0.3,0.7,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(0.7,0.3,0.3);
 Side.VArray[1] = CreateVertex(0.7, 0.3, 0.7);
 Side.VArray[2] = CreateVertex(0.7, 0.7, 0.7);
 Side.VArray[3] = CreateVertex(0.7,0.7,0.3);
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      
 
 return newList;
}

klist CreateBox()
{
  klist newList = CreateEmptyList();

  kface Side1;
  Side1.NumVertices = 4;
  Side1.VArray[0] = CreateVertex(0.05,0.05,0.05);
  Side1.VArray[1] = CreateVertex(0.95, 0.05, 0.05);
  Side1.VArray[2] = CreateVertex(0.95, 0.05, 0.95);
  Side1.VArray[3] = CreateVertex(0.05, 0.05, 0.95); 
  Side1 = CalculateNormals(Side1);
  newList = addNode(newList, Side1);   

  kface Side2;
  Side2.NumVertices = 4;
  Side2.VArray[0] = CreateVertex(0.05, 0.05, 0.95); 
  Side2.VArray[1] = CreateVertex(0.95, 0.05, 0.95);
  Side2.VArray[2] = CreateVertex(0.95, 0.95, 0.95);
  Side2.VArray[3] = CreateVertex(0.05, 0.95, 0.95);  
  Side2 = CalculateNormals(Side2);
  newList = addNode(newList, Side2);   

  kface Side3;
  Side3.NumVertices = 4;
  Side3.VArray[0] = CreateVertex(0.05,0.05,0.05);
  Side3.VArray[1] = CreateVertex(0.95, 0.05, 0.05);  
  Side3.VArray[2] = CreateVertex(0.95, 0.95, 0.05);
  Side3.VArray[3] = CreateVertex(0.05, 0.95, 0.05);    
  Side3 = CalculateNormals(Side3);
  newList = addNode(newList, Side3);   

  kface Side4;
  Side4.NumVertices = 4;
  Side4.VArray[0] = CreateVertex(0.05, 0.95, 0.05);    
  Side4.VArray[1] = CreateVertex(0.95, 0.95, 0.05);
  Side4.VArray[2] = CreateVertex(0.95, 0.95, 0.95);
  Side4.VArray[3] = CreateVertex(0.05, 0.95, 0.95);  
  Side4 = CalculateNormals(Side4);
  newList = addNode(newList, Side4);   
  
  kface Side5;
  Side5.NumVertices = 4;
  Side5.VArray[0] = CreateVertex(0.95, 0.95, 0.05);
  Side5.VArray[1] = CreateVertex(0.95, 0.95, 0.95);  
  Side5.VArray[2] = CreateVertex(0.95, 0.05, 0.95);
  Side5.VArray[3] = CreateVertex(0.95, 0.05, 0.05);
  Side5 = CalculateNormals(Side5);
  newList = addNode(newList, Side5);   

  kface Side6;
  Side6.NumVertices = 4;
  Side6.VArray[0] = CreateVertex(0.05, 0.95, 0.05); 
  Side6.VArray[1] = CreateVertex(0.05, 0.95, 0.95);    
  Side6.VArray[2] = CreateVertex(0.05, 0.05, 0.95);   
  Side6.VArray[3] = CreateVertex(0.05,0.05,0.05);  
  Side6 = CalculateNormals(Side6);
  newList = addNode(newList, Side6);  
  
  return newList;  
}
klist CreateWallNorth()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.0, 0.1, 1.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.1, 1.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.9, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.97); 
 Side.VArray[1] = CreateVertex(1.0, 0.1, 0.97);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 0.97);   
 Side.VArray[3] = CreateVertex(0.0, 0.9, 0.97);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0, 0.9, 0.97); 
 Side.VArray[1] = CreateVertex(1.0, 0.9, 0.97);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.9, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.97); 
 Side.VArray[1] = CreateVertex(0.0, 0.9, 0.97);    
 Side.VArray[2] = CreateVertex(0.0, 0.9, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(1.0, 0.1, 0.97); 
 Side.VArray[1] = CreateVertex(1.0, 0.9, 0.97);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 1.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     
 
 return newList;      
}
klist CreateWallEast()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;   
 Side.VArray[0] = CreateVertex(1.0, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.899, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.97, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(0.97, 0.899, 0.0);    
 Side.VArray[2] = CreateVertex(0.97, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(0.97, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.97, 0.899, 0.0); 
 Side.VArray[1] = CreateVertex(0.97, 0.899, 1.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.899, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.97, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(0.97, 0.899, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.899, 0.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.97, 0.1, 1.0); 
 Side.VArray[1] = CreateVertex(0.97, 0.899, 1.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);         
  
 return newList;   
}
klist CreateWallSouth()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.1, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 0.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.9, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 

 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.03); 
 Side.VArray[1] = CreateVertex(1.0, 0.1, 0.03);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 0.03);   
 Side.VArray[3] = CreateVertex(0.0, 0.9, 0.03);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0, 0.9, 0.03); 
 Side.VArray[1] = CreateVertex(1.0, 0.9, 0.03);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 0.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.9, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.03); 
 Side.VArray[1] = CreateVertex(0.0, 0.9, 0.03);    
 Side.VArray[2] = CreateVertex(0.0, 0.9, 0.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(1.0, 0.1, 0.03); 
 Side.VArray[1] = CreateVertex(1.0, 0.9, 0.03);    
 Side.VArray[2] = CreateVertex(1.0, 0.9, 0.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     
 
 return newList;      
}

klist CreateWallWest()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(0.0, 0.899, 0.0);    
 Side.VArray[2] = CreateVertex(0.0, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.03, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(0.03, 0.899, 0.0);    
 Side.VArray[2] = CreateVertex(0.03, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(0.03, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.03, 0.899, 0.0); 
 Side.VArray[1] = CreateVertex(0.03, 0.899, 1.0);    
 Side.VArray[2] = CreateVertex(0.0, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.899, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    

 Side.VArray[0] = CreateVertex(0.03, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(0.03, 0.899, 0.0);    
 Side.VArray[2] = CreateVertex(0.0, 0.899, 0.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.03, 0.1, 1.0); 
 Side.VArray[1] = CreateVertex(0.03, 0.899, 1.0);    
 Side.VArray[2] = CreateVertex(0.0, 0.899, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    
 
 return newList;      
}
klist CreateWallBottom()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.0, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.0, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(0.0, 0.0, 1.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.0, 1.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.1, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);     

 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.1, 0.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0); 
 Side.VArray[1] = CreateVertex(0.0, 0.0, 1.0);    
 Side.VArray[2] = CreateVertex(0.0, 0.1, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(1.0, 0.0, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.0, 1.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.1, 1.0);   
 Side.VArray[3] = CreateVertex(1.0, 0.1, 0.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);      

 Side.VArray[0] = CreateVertex(0.0, 0.1, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.1, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.1, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.1, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 return newList;      
}

klist CreateWallBounds()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4; 
 Side.VArray[0] = CreateVertex(-0.002, 0.0, 1.002); 
 Side.VArray[1] = CreateVertex(-0.002, -1.6, 1.002);    
 Side.VArray[2] = CreateVertex(10.002, -1.6, 1.002);   
 Side.VArray[3] = CreateVertex(10.002, 0.0, 1.002);   
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 Side.VArray[0] = CreateVertex(-0.002, 0.0, -9.002); 
 Side.VArray[1] = CreateVertex(-0.002, -1.6, -9.002);    
 Side.VArray[2] = CreateVertex(10.002, -1.6, -9.002);   
 Side.VArray[3] = CreateVertex(10.002, 0.0, -9.002); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(-0.002, 0.0, -9.002); 
 Side.VArray[1] = CreateVertex(-0.002, -1.6, -9.002);    
 Side.VArray[2] = CreateVertex(-0.002, -1.6, 1.002);   
 Side.VArray[3] = CreateVertex(-0.002, 0.0, 1.002); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);   

 Side.VArray[0] = CreateVertex(10.002, 0.0, -9.002); 
 Side.VArray[1] = CreateVertex(10.002, -1.6, -9.002);    
 Side.VArray[2] = CreateVertex(10.002, -1.6, 1.002);   
 Side.VArray[3] = CreateVertex(10.002, 0.0, 1.002); 
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);    
 
 return newList;           
}

klist CreateHole()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4;
 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0); 
 Side.VArray[1] = CreateVertex(1.0, 0.0, 0.0);    
 Side.VArray[2] = CreateVertex(1.0, 0.0, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.0, 1.0);  
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side); 
 
 return newList;      
}

klist CreateMenus()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4; 
 Side.VArray[0] = CreateVertex(0.0, 0.0, 1.0); 
 Side.VArray[1] = CreateVertex(1.1, 0.0, 1.0);    
 Side.VArray[2] = CreateVertex(1.1, 0.85, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.85, 1.0);   
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 return newList;     
}

klist CreateBackground()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4; 
 Side.VArray[0] = CreateVertex(0.0, 0.0, 0.0); 
 Side.VArray[1] = CreateVertex(120.0, 0.0, 0.0);    
 Side.VArray[2] = CreateVertex(120.0, 80.0, 0.0);   
 Side.VArray[3] = CreateVertex(0.0, 80.0, 0.0);   
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  
 
 return newList;
}

klist CreateHUD()
{
 klist newList = CreateEmptyList();
 kface Side;
 Side.NumVertices = 4; 
 Side.VArray[0] = CreateVertex(0.0, 0.0, 1.0); 
 Side.VArray[1] = CreateVertex(0.138, 0.0, 1.0);    
 Side.VArray[2] = CreateVertex(0.138, 0.138, 1.0);   
 Side.VArray[3] = CreateVertex(0.0, 0.138, 1.0);   
 Side = CalculateNormals(Side);
 newList = addNode(newList, Side);  

 return newList;
}
