#include "Painter.h"
#include "Particles.h"
#include "Picking.h"
#include "Menu.h"

//One thing you'll notice right away is my decision to hardcode absolutely every graphics
//object.  I don't use OBJ files or anything.  I did this for a few reasons...
//1) I wanted very specific objects in my game.
//2) I wanted to play around with the GL functions.
//3) I don't have any software to make OBJ files.
//4) I actually made my own filetype called KOB.  Malloc didn't like it and I basically lost 3 days
//   of work.  So...we're not doing that again.

//So now I use KLists.  They are basically Quads with defined vertices.  To see all my
//pre-defined vertices, go check out Faces.c.
//More complicated objects, I relented and used Glut spheres and cylinders.

//You'll notice that I don't use blending or check for transparencies in textures.
//I didn't go that route.  I took the HUD and particle systems as my two extras.
//Hopefully they're good enough to count for a decent pass to this project.

//I define an int to reference every texture.
GLuint Thvrim;   //The green rim on an HV Block.
GLuint Thvbox;   //The 4 arrows.
GLuint Tvrim;    //The blue rim on the V Block.
GLuint Tvbox;    //The 2 arrows pointing up/down.
GLuint Thrim;    //Orange rim on the H Block.
GLuint Thbox;   //Arrows pointing appropriately.
GLuint Tdbox;   //The spider on the Death Block.
GLuint Tdrim;   //Black rim on a death block.
klist BoxList;  //The list of predefined vertices for a simple box.
klist BoxRimList; //The list of predefined vertices for a box wireframe.

GLuint Tbackg;  //The background image for World 1.
klist BackgroundList; //The one Quad used to paint it.

GLuint Tsuccess;  //Success message.
GLuint Tfailure;  //Failure message.
GLuint Tgmenu;    //In game menu.
GLuint Ttitle;    //Title screen.
GLuint Thelp1;    //Help screen.
GLuint Thelp2;    //Another help screen.
GLuint Thelp3;    //The last one.
GLuint Tworld1;   //The world picture.  Similar to background, but places levels down.
GLuint Tmcurrent; //The current 2D screen being shown to the player.  This is only for out-of-game menus.
GLuint Tlice;   //My ice picker for the front screen.
GLuint Tlpid;   //The little Pid for the world map.
klist MenuList; //The quad used to render my menu screens.

GLuint Thud1;  //I split up the Hud into 6 equally sized pieces.
GLuint Thud2;
GLuint Thud3;
GLuint Thud4;
GLuint Thud5;
GLuint Thud6;
GLuint Tlnothing; //This is my icon for when no object is selected.
GLuint Tlpenguin;  //The rest of these are icons for the selected object.
GLuint Tlcrystal;
GLuint Tlexit;
GLuint Tlgolem;
GLuint Tlfury;
GLuint Tlswitch;
GLuint Tltrap;
GLuint Tlarrow;
GLuint Tlbox;
GLuint Tlcurrent;  //The currently selected object by the picker.
klist HUDList;   //My list to render the HUD and the object selected displayed on it.

GLuint Tbottom;  //The icy bottom to the board.
GLuint Tdirt;    //Dirt on a spot.
GLuint Thole;
GLuint Twall;    //The walls for the level.
klist WallBottomList;  //The list of vertices for the bottom.
klist WallBoundsList;
klist HoleList;
klist WallNorthList;
klist WallSouthList;
klist WallEastList;  //All of these are similar.  Draw them all and you have an open box.
klist WallWestList;

GLuint Tfury;  //My flaming textures.
GLuint Tflame;
GLuint Tpfire;
GLuint Tgolem;  //The golems.  A crappy statue texture that I should probably change someday.
GLuint Tgolemends;
klist GolemEndsList;  //The top and bottom ends to my golem.
GLUquadricObj* QuadList;  //This quadlist is used to calculate texture co-ords for my more
                          //complicated objects, like spheres and such.

GLuint Tpidfront;
GLuint Tpidblack;
GLuint Tpidfeet;  //My textures for pid.
GLuint Tpidface;
klist PidFaceList;  //My vertex list for each part of Pid.  I have different lists, because
klist PidBodyList;  //I set it so that each list may have only one texture associated with it.
klist PidFrontList;
klist PidFeetList;

GLuint Ttrap;    //My trapmine texture.
GLuint Ttrapedge;  //The edges sticking out of it.
klist TrapList;
klist TrapEdgeList;  //The two lists for the trapmine.

//If you haven't guessed yet, it goes on like this for awhile still.

GLuint Tswitchbox;
GLuint Tredswitch;
GLuint Tyellowswitch;
klist SwitchBoxList;
klist SwitchList;

GLuint Tcrystal;
klist CrystalList;

GLuint Texit;
klist ExitList;

GLuint Tarrow;
klist ArrowStemList;
klist ArrowTopList;
GLuint Tarrowrim;
klist ArrowRimList;

TextureImage Tcursor;

//Okay, here we go.
char* Info = "";  //This string is what is currently displayed in the info box.
int ColorCode = 0; //This is an indicator used to determine if we're in picking mode or not.
                // The colors used to draw objects will vary depending on the answer.
ColourType Active = ISRED;  //Sets the active colour for the level.  Always starts on Red.
bool Rotate = false;  //This slows down the rotation speed of the crystals.
bool Skidmark = false;
float RotateCrystal = 0.0;


bool LoadTGA(TextureImage *texture, char *filename)	// Loads A TGA File Into Memory
{    
	GLubyte		TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};// Uncompressed TGA Header
	GLubyte		TGAcompare[12];							// Used To Compare TGA Header
	GLubyte		header[6];						  	    // First 6 Useful Bytes From The Header
	GLuint		bytesPerPixel;							// Holds Number Of Bytes Per Pixel Used In The TGA File
	GLuint		imageSize;							// Used To Store The Image Size When Setting Aside Ram
	GLuint		temp;								// Temporary Variable
	GLuint		type=GL_RGBA;							// Set The Default GL Mode To RBGA (32 BPP)

	FILE *file = fopen(filename, "rb");						// Open The TGA File

	if(	file==NULL ||								// Does File Even Exist?
		fread(TGAcompare,1,sizeof(TGAcompare),file)!=sizeof(TGAcompare) ||	// Are There 12 Bytes To Read?
		memcmp(TGAheader,TGAcompare,sizeof(TGAheader))!=0		||	// Does The Header Match What We Want?
		fread(header,1,sizeof(header),file)!=sizeof(header))			// If So Read Next 6 Header Bytes
	{
		if (file == NULL)							// Did The File Even Exist? *Added Jim Strong*
        {
            printf("File not found.");
			return false;							// Return False
        }
		else
		{
            printf("File screwed up.");
			fclose(file);							// If Anything Failed, Close The File
			return false;							// Return False
		}
	}
	texture->width  = header[1] * 256 + header[0];					// Determine The TGA Width	(highbyte*256+lowbyte)
	texture->height = header[3] * 256 + header[2];					// Determine The TGA Height	(highbyte*256+lowbyte)
    
 	if(	texture->width	<=0	||						// Is The Width Less Than Or Equal To Zero
		texture->height	<=0	||						// Is The Height Less Than Or Equal To Zero
		(header[4]!=24 && header[4]!=32))					// Is The TGA 24 or 32 Bit?
	{
        printf("File doesn't fit header.");
		fclose(file);								// If Anything Failed, Close The File
		return false;								// Return False
	}

    texture->bpp	= header[4];							// Grab The TGA's Bits Per Pixel (24 or 32)
	bytesPerPixel	= texture->bpp/8;						// Divide By 8 To Get The Bytes Per Pixel
	imageSize	= texture->width*texture->height*bytesPerPixel;			// Calculate The Memory Required For The TGA Data

	texture->imageData=(GLubyte *)malloc(imageSize);				// Reserve Memory To Hold The TGA Data

	if(	texture->imageData==NULL ||						// Does The Storage Memory Exist?
		fread(texture->imageData, 1, imageSize, file)!=imageSize)		// Does The Image Size Match The Memory Reserved?
	{
		if(texture->imageData!=NULL)						// Was Image Data Loaded
			free(texture->imageData);					// If So, Release The Image Data

		fclose(file);								// Close The File
		return false;								// Return False
	} 

	for(GLuint i=0; i<int(imageSize); i+=bytesPerPixel)				// Loop Through The Image Data
	{										// Swaps The 1st And 3rd Bytes ('R'ed and 'B'lue)
		temp=texture->imageData[i];						// Temporarily Store The Value At Image Data 'i'
		texture->imageData[i] = texture->imageData[i + 2];			// Set The 1st Byte To The Value Of The 3rd Byte
		texture->imageData[i + 2] = temp;					// Set The 3rd Byte To The Value In 'temp' (1st Byte Value)
	}

	fclose (file);									// Close The File

	// Build A Texture From The Data
	glGenTextures(1, &texture[0].texID);						// Generate OpenGL texture IDs

	glBindTexture(GL_TEXTURE_2D, texture[0].texID);					// Bind Our Texture
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);		// Linear Filtered
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);		// Linear Filtered
	if (texture[0].bpp==24)								// Was The TGA 24 Bits
	{
		type=GL_RGB;								// If So Set The 'type' To GL_RGB
	}
	glTexImage2D(GL_TEXTURE_2D, 0, type, texture[0].width, texture[0].height, 0, type, GL_UNSIGNED_BYTE, texture[0].imageData);

	return true;									// Texture Building Went Ok, Return True
}

//This function loads a RAW file and prepares it as a texture.
//This function was taken directly from NEHE's page.  I only made
//changes so that it would be UNIX-friendly.
GLuint LoadTexture( const char * filename, int pwidth, int pheight )
{
    int wrap = 1;
    GLuint texture;
    int width, height;
    unsigned char* data;
    FILE * file;

    // open texture data
    file = fopen( filename, "rb" );
    if ( file == NULL ) 
    {
        printf("Could not load texture %s\r\n", filename);
        exit(1);
    }

    // allocate buffer
    width = pwidth;
    height = pheight;
    data = (unsigned char*) malloc( width * height * 3 );

    // read texture data

    fread( data, width * height * 3, 1, file );
    fclose( file );
    // allocate a texture name
    glGenTextures( 1, &texture );

    // select our current texture
    glBindTexture( GL_TEXTURE_2D, texture );

    // select modulate to mix texture with color for shading
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    // when texture area is small, bilinear filter the closest mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_LINEAR_MIPMAP_NEAREST );
    // when texture area is large, bilinear filter the first mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    // if wrap is true, the texture wraps over at the edges (repeat)
    //       ... false, the texture ends at the edges (clamp)
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
                    wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
                     wrap ? GL_REPEAT : GL_CLAMP );

    // build our texture mipmaps
    gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width, height,
                       GL_RGB, GL_UNSIGNED_BYTE, data );
    // free buffer
    free( data );

    return texture;      
}

//This function paints the Klist with the texture given.
void PaintObject(klist theList, GLuint texture, GLenum Type, ColourType Col, bool NoLighting, double Alpha)
{
 if(ColorCode == 1) //If we're in picking mode...
 {
  glDisable( GL_TEXTURE_2D ); //Disable texturing and draw a gray representation of the object.
  glColor3ub(theList.ColorCode, theList.ColorCode, theList.ColorCode);    //The gray representation
  //of the object is stored in the klist associated with it.     
 }
 else
 {
  glEnable( GL_TEXTURE_2D ); //I enable Texturing here.  I always do it here, in case it was
                             //disabled by the last object.  It's here for the worst-case scenario.
  glBindTexture( GL_TEXTURE_2D, texture);  //Draw with the texture given.
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);  //This is a default option.
  if(NoLighting) glColor3f(1, 1, 1); //If lighting doesn't affect this object, make it as bright as possible.
  else glColor4f(0.7, 0.7, 0.7, Alpha); //Otherwise shade it slightly so that lighting can still affect it.
  glShadeModel(GL_SMOOTH);  
  float NewAlpha = 1.0;
   if(Col != Active && Col != ISGREY) //If this object is currently inactive...
   {
     glBlendFunc(GL_ONE, GL_ONE);
     glDisable( GL_TEXTURE_2D );  //Turn off the textures.
     NewAlpha -= 0.5;
   }
   if(Col == ISYELLOW)  //Colour the object based on it's colour.
     glColor4f(1, 1, 0.0, NewAlpha);
   else if(Col == ISRED) glColor4f(1, 0.0, 0.0, NewAlpha);
  }
  //Begin with the type specified.
  glBegin(Type);
  theList = ResetList(theList);
  while(theList.Current != NULL)
  {
    //The normals are calculated and stored in the klist, so simply retrieve them.
    //My normals are per-face, not per-vertex.  I personally think per-face looks better
    //given my artstyle for the game.
    glNormal3f(theList.Current->theFace.NormalX, theList.Current->theFace.NormalY, theList.Current->theFace.NormalZ);
    //I don't automatically generate texture coordinates.  Instead I flip the textures.
    //This is because I drew my textures in Flipped RAW format.  So here I just flip them back.
    float texX = 0; float texY = 0;   
    for(int k = 0; k < theList.Current->theFace.NumVertices; k++)
     {       
       glTexCoord2f (texX,texY);
       if(texX == 0 && texY == 0)
         texX++;
       else if(texY == 0 && texX == 1)
         texY++;
       else if(texX && texY) texX--;
       glVertex3f(theList.Current->theFace.VArray[k].X, theList.Current->theFace.VArray[k].Y, -theList.Current->theFace.VArray[k].Z);
//       printf("%d %d %d", theList.Current->theFace.VArray[k].X, theList.Current->theFace.VArray[k].Y, -theList.Current->theFace.VArray[k].Z);
     }
     theList.Current = theList.Current->next;
   }       
 glEnd();
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);   
}

//This function loads ALL of the textures for the game.
//If you're wondering about the width/height of a raw file, you'll find it listed here.
void LoadGraphics()
{
 Thvrim = LoadTexture("Graphics/hvrim.raw", 128, 128);
 Thvbox = LoadTexture("Graphics/hv.raw", 128, 128);
 Tvrim = LoadTexture("Graphics/vrim.raw", 128, 128);
 Tvbox = LoadTexture("Graphics/v.raw", 128, 128);
 Thrim = LoadTexture("Graphics/hrim.raw", 128, 128);
 Thbox = LoadTexture("Graphics/h.raw", 128, 128);  
 Tdrim = LoadTexture("Graphics/drim.raw", 128, 128);
 Tdbox = LoadTexture("Graphics/d.raw", 128, 128);   
 Tcrystal = LoadTexture("Graphics/crystal.raw", 128, 128);   
 Tarrow = LoadTexture("Graphics/arrow.raw", 128, 128);  
 Tarrowrim = LoadTexture("Graphics/arim.raw", 128, 128);  
 Texit = LoadTexture("Graphics/exit.raw", 128, 128); 
 Ttrap = LoadTexture("Graphics/trap.raw", 128, 128); 
 Ttrapedge = LoadTexture("Graphics/edge.raw", 128, 128);  
 Tswitchbox = LoadTexture("Graphics/switch.raw", 128, 128);   
 Tredswitch = LoadTexture("Graphics/red.raw", 128, 128);   
 Tyellowswitch = LoadTexture("Graphics/yellow.raw", 128, 128);  
 Tpidfront = LoadTexture("Graphics/front.raw", 128, 128);
 Tpidblack = LoadTexture("Graphics/black.raw", 128, 128);
 Tpidface = LoadTexture("Graphics/face.raw", 128, 128); 
 Tpidfeet = LoadTexture("Graphics/feet.raw", 128, 128);
 Tfury = LoadTexture("Graphics/fury.raw", 128, 128); 
 Tflame = LoadTexture("Graphics/flame.raw", 128, 128);  
 Tgolem = LoadTexture("Graphics/golem.raw", 128, 128);   
 Tgolemends = LoadTexture("Graphics/golemends.raw", 128, 128);
 Tbottom = LoadTexture("Graphics/bottom.raw", 128, 128);
 Tdirt = LoadTexture("Graphics/dirt.raw", 128, 128); 
 Twall = LoadTexture("Graphics/wall.raw", 128, 128);
 Tbackg = LoadTexture("Graphics/backg.raw", 512, 512);
 Thud1 = LoadTexture("Graphics/hud1.raw", 128, 128);
 Thud2 = LoadTexture("Graphics/hud2.raw", 128, 128);
 Thud3 = LoadTexture("Graphics/hud3.raw", 128, 128);
 Thud4 = LoadTexture("Graphics/hud4.raw", 128, 128);
 Thud5 = LoadTexture("Graphics/hud5.raw", 128, 128);
 Thud6 = LoadTexture("Graphics/hud6.raw", 128, 128);
 Tlnothing = LoadTexture("Graphics/lnothing.raw", 64, 64);
 Tlcrystal = LoadTexture("Graphics/lcrystal.raw", 64, 64);
 Tlpenguin = LoadTexture("Graphics/lpenguin.raw", 64, 64);  
 Tlexit = LoadTexture("Graphics/lexit.raw", 64, 64); 
 Tlgolem = LoadTexture("Graphics/lgolem.raw", 64, 64);  
 Tlfury = LoadTexture("Graphics/lfury.raw", 64, 64);   
 Tlswitch = LoadTexture("Graphics/lswitch.raw", 64, 64);  
 Tltrap = LoadTexture("Graphics/ltrap.raw", 64, 64);   
 Tlarrow = LoadTexture("Graphics/larrow.raw", 64, 64);   
 Tlbox = LoadTexture("Graphics/lbox.raw", 64, 64);   
 Ttitle = LoadTexture("Graphics/title.raw", 800, 600); 
 Thelp1 = LoadTexture("Graphics/Help1.raw", 800, 600); 
 Thelp2 = LoadTexture("Graphics/Help2.raw", 800, 600); 
 Thelp3 = LoadTexture("Graphics/Help3.raw", 800, 600);    
 Tworld1 = LoadTexture("Graphics/world1.raw", 800, 600);
 Tsuccess = LoadTexture("Graphics/success.raw", 512, 182);
 Tfailure = LoadTexture("Graphics/fail.raw", 512, 182); 
 Tgmenu = LoadTexture("Graphics/gmenu.raw", 512, 182);  
 Tmcurrent = Ttitle;
 Tlpid = LoadTexture("Graphics/lpid.raw", 32, 32);
 Tlice = LoadTexture("Graphics/lice.raw", 32, 32);
 Tpfire = LoadTexture("Graphics/pfire.raw", 64, 64);
 InitializeSystems(Tpfire);
 Thole = LoadTexture("Graphics/hole.raw", 4, 4); 
 LoadTGA(&Tcursor,"Graphics/cursor.tga");
 MenuList = CreateMenus();
 Tlcurrent = Tlnothing;
//Set up the particle systems.
 InitializeFire();
 InitializeExit();
 InitializeLights();
 InitializeSkidMark();
//Set up the Klists for each object and then assign it a color code for picking purposes. 
 HUDList = CreateHUD();   
 HUDList.ColorCode = 0;
 BackgroundList = CreateBackground();
 BackgroundList.ColorCode = 0;
 WallNorthList = CreateWallNorth();
 WallNorthList.ColorCode = 0;
 WallEastList = CreateWallEast();
 WallEastList.ColorCode = 0; 
 WallSouthList = CreateWallSouth();
 WallSouthList.ColorCode = 0; 
 WallWestList = CreateWallWest();
 WallWestList.ColorCode = 0; 
 WallBottomList = CreateWallBottom();
 WallBottomList.ColorCode = 0; 
 WallBoundsList = CreateWallBounds();
 WallBoundsList.ColorCode = 0;
 HoleList = CreateHole();
 HoleList.ColorCode = 0; 
 GolemEndsList = CreateGolemEnds(); 
 GolemEndsList.ColorCode = PGOLEM;
 QuadList = gluNewQuadric();			// Create a pointer to the Quadric object.
 gluQuadricNormals(QuadList, GLU_SMOOTH);	
 gluQuadricTexture(QuadList, GL_TRUE);	// It will calculate fractional co-ords for the object.
 PidFaceList = CreatePidFace();
 PidFaceList.ColorCode = PPID;
 PidBodyList = CreatePidBody();
 PidBodyList.ColorCode = PPID;
 PidFrontList = CreatePidFront();
 PidFrontList.ColorCode = PPID;
 PidFeetList = CreatePidFeet();
 PidFeetList.ColorCode = PPID;
 SwitchBoxList = CreateSwitchBox();
 SwitchBoxList.ColorCode = PSWITCH;
 SwitchList = CreateSwitch();
 SwitchList.ColorCode = PSWITCH; 
 ExitList = CreateExit();
 ExitList.ColorCode = PEXIT; 
 TrapList = CreateTrap();
 TrapList.ColorCode = PTRAP;
 TrapEdgeList = CreateTrapEdge();
 TrapEdgeList.ColorCode = PTRAP;
 ArrowStemList = CreateArrowStem();
 ArrowStemList.ColorCode = PARROW;
 ArrowTopList = CreateArrowTop();
 ArrowTopList.ColorCode = PARROW; 
 ArrowRimList = CreateArrowRim(); 
 ArrowRimList.ColorCode = PARROW; 
 BoxList = CreateBox();
 BoxList.ColorCode = PBOX; 
 BoxRimList = CreateBoxRim();
 BoxRimList.ColorCode = PBOX;  
 CrystalList = CreateCrystal();
 CrystalList.ColorCode = PCRYSTAL; 
}

//The following functions are for each object.  The functions just paint all the
//klists associated with it.  If it gets more complicated than that, I'll document it.
//The colour passed is the colour for the tile we're painting.
void PaintHVBlock(ColourType Colour)
{ 
 PaintObject(BoxList, Thvbox, GL_QUADS, Colour, false, 1.0);
 PaintObject(BoxRimList, Thvrim, GL_QUADS, Colour, false, 1.0);  
}

void PaintVBlock(ColourType Colour)
{ 
 PaintObject(BoxList, Tvbox, GL_QUADS, Colour, false, 1.0);
 PaintObject(BoxRimList, Tvrim, GL_QUADS, Colour, false, 1.0);       
}

void PaintHBlock(ColourType Colour)
{
 PaintObject(BoxList, Thbox, GL_QUADS, Colour, false, 1.0);
 PaintObject(BoxRimList, Thrim, GL_QUADS, Colour, false, 1.0);   
}

void PaintDBlock(ColourType Colour)
{
 PaintObject(BoxList, Tdbox, GL_QUADS, Colour, false, 1.0);
 PaintObject(BoxRimList, Tdrim, GL_QUADS, Colour, false, 1.0);    
}

void PaintCrystal(ColourType Colour)
{
 if(ColorCode != 1) glBlendFunc(GL_SRC_COLOR, GL_SRC_COLOR);
 glPushMatrix();
 glTranslatef(0.5, 0.5, -0.5);
 if(Rotate) 
 {
  RotateCrystal += 0.8;
  Rotate = false;
 }
 else Rotate = true;
 if(RotateCrystal > 360) RotateCrystal = 0;
 glRotatef(RotateCrystal, 0, 1, 0);
 glTranslatef(-0.5, -0.5, 0.5);
 PaintObject(CrystalList, Tcrystal, GL_TRIANGLES, Colour, false, 0.7);
 glPopMatrix();
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
}

void PaintArrow(ColourType Colour)
{
 PaintObject(ArrowStemList, Tarrow, GL_QUADS, Colour, false, 1.0); 
 PaintObject(ArrowTopList, Tarrow, GL_TRIANGLES, Colour, false, 1.0);
 PaintObject(ArrowRimList, Tarrowrim, GL_QUADS, Colour, false, 1.0);
}

void PaintExit(ColourType Colour)
{
 PaintObject(ExitList, Texit, GL_QUADS, Colour, false, 1.0);
}

void PaintOpenExit(ColourType Colour)
{
 PaintObject(ExitList, Texit, GL_QUADS, Colour, false, 1.0);
}

void PaintRedSwitch(ColourType Colour)
{    
 PaintObject(SwitchBoxList, Tswitchbox, GL_QUADS, ISGREY, false, 1.0);
 PaintObject(SwitchList, Tredswitch, GL_TRIANGLES, ISGREY, false, 1.0);
}

void PaintYellowSwitch(ColourType Colour)
{
 PaintObject(SwitchBoxList, Tswitchbox, GL_QUADS, ISGREY, false, 1.0);
 PaintObject(SwitchList, Tyellowswitch, GL_TRIANGLES, ISGREY, false, 1.0);
}

void PaintTrap(ColourType Colour)
{
 PaintObject(TrapList, Ttrap, GL_QUADS, Colour, false, 1.0);
 PaintObject(TrapEdgeList, Ttrapedge, GL_QUADS, Colour, false, 1.0);
}

void PaintPid(ColourType Colour, bool Anim)
{
  PaintObject(PidFaceList, Tpidface, GL_QUADS, Colour, false, 1.0);    
  PaintObject(PidBodyList, Tpidblack, GL_QUADS, Colour, false, 1.0);      
  PaintObject(PidFrontList, Tpidfront, GL_QUADS, Colour, false, 1.0);
  PaintObject(PidFeetList, Tpidfeet, GL_QUADS, Colour, false, 1.0);
  if(Skidmark)
   Skidmark = PaintSkidMark(Anim);
}

void PaintAlly(ColourType Colour)
{
  PaintObject(PidFaceList, Tpidface, GL_QUADS, Colour, false, 1.0);    
  PaintObject(PidBodyList, Tpidblack, GL_QUADS, Colour, false, 1.0);     
  PaintObject(PidFrontList, Tpidfront, GL_QUADS, Colour, false, 1.0);
  PaintObject(PidFeetList, Tswitchbox, GL_QUADS, Colour, false, 1.0);     
}

//Furies and Flames use a Sphere as the base.  So I rotate the sphere based
//on the texture map and then draw it.  
void PaintFury(ColourType Colour)
{
 if(ColorCode == 1) 
 {
  glDisable( GL_TEXTURE_2D);                 
  glColor3ub(PFURY, PFURY, PFURY);     
 }
 else
 {
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, Tfury);     
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glColor3f(0.7, 0.7, 0.7);
  glShadeModel(GL_SMOOTH);
 }
 glPushMatrix();
 glTranslatef(0.5, 0.5, -0.5);
 glRotatef(-90, 1, 0, 0);
 glRotatef(180, 0, 1, 0);  
 gluSphere(QuadList,0.48,15,15);
 glPopMatrix();
 glPushMatrix();
 glPopMatrix();
}

void PaintFlame(ColourType Colour)
{
 if(ColorCode == 1) 
 {
  glDisable( GL_TEXTURE_2D);              
  glColor3ub(PFLAME, PFLAME, PFLAME);
 }
 else
 {
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, Tflame);     
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glColor3f(0.7, 0.7, 0.7);
  glShadeModel(GL_SMOOTH);  
 }
 
 glPushMatrix();
 glTranslatef(0.5, 0.5, -0.5);
 glRotatef(-90, 1, 0, 0);
 glRotatef(180, 0, 1, 0); 
 gluSphere(QuadList,0.48,15,15);
 glPopMatrix();
 glPushMatrix();
 glPopMatrix();
}


//Golems use a cylinder between the two end pieces.
void PaintGolem(ColourType Colour)
{
 PaintObject(GolemEndsList, Tgolemends, GL_QUADS, ISGREY, false, 1.0);
 if(ColorCode == 1) 
 {
   glDisable( GL_TEXTURE_2D);
   glColor3ub(PGOLEM, PGOLEM, PGOLEM);
 }
 else
 {
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, Tgolem);     
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glColor3f(0.7, 0.7, 0.7);
  glShadeModel(GL_SMOOTH);
 }
 glPushMatrix();
 glTranslatef(0.5, 0.05, -0.5); 
 glRotatef(-90, 1, 0, 0); 
 gluCylinder(QuadList, 0.48, 0.40, 0.9, 10, 10);
 glPopMatrix();
}

//Paints the winning screen.  The functions are pretty much self-explanatory from here.
void PaintWinner()
{
 glPushMatrix();
 glTranslatef(-0.35, 0.12, 0);
 glScalef(5.0, 2.0, 1.0);
 PaintObject(HUDList, Tsuccess, GL_QUADS, ISGREY, true, 1.0);
 glPopMatrix();         
}

void PaintGameMenu()
{
 glPushMatrix();
 glTranslatef(-0.35, -0.05, 0);
 glScalef(5.0, 1.2, 1.0);
 PaintObject(HUDList, Tgmenu, GL_QUADS, ISGREY, true, 1.0);
 glPopMatrix();   
 PaintHelp(0);         
}


void PaintLoser()
{
 glPushMatrix();
 glTranslatef(-0.35, 0.12, 0);
 glScalef(5.0, 2.0, 1.0);
 PaintObject(HUDList, Tfailure, GL_QUADS, ISGREY, true, 1.0);
 glPopMatrix(); 
 PaintGameMenu();        
}


void PaintBackground()
{
 glPushMatrix();
 glTranslatef(-60, -40, -90);
 PaintObject(BackgroundList, Tbackg, GL_QUADS, ISGREY, true, 1.0);
 glPopMatrix();     
}

//This paints a string.  It'll actually account for newlines, making me pretty happy.
//I did make a few changes, but it's actually the paintstring function which I got
//from the OpenGL Lighthouse tutorial.
void PaintString(double x, double y, void* font, char* string)
{  
  char* c;

  glRasterPos3f(x, y, 0.9999);
  glDisable(GL_BLEND);
  glEnable(GL_TEXTURE_2D);
  glColor3f(0.0, 0.0, 0.0);
  
  for (c = string; *c != '\0'; c++) 
  {      
   if(*c == '_') *c = ' ';
    glutBitmapCharacter(font, *c);
    if(*c == '\n') 
    {
     y = y+15;
     glRasterPos3f(x, y, 0.9999);
    }
  }
}

//Okay, this is admittedly kinda messy.  It retrieves the current screen from
//Menu.h and sets the current screen appropriately.

//It then paints that screen and puts down any icon based on the screen.  So
//I have a little ice icon for the main screen and a little penguin for the
//world map screen.  The position of these icons are defined by Menu.h.
void PaintTitle()
{
 if(GetScreen() == STITLE) Tmcurrent = Ttitle;
 else if(GetScreen() == SHELP1) Tmcurrent = Thelp1;
 else if(GetScreen() == SHELP2) Tmcurrent = Thelp2;
 else if(GetScreen() == SHELP3) Tmcurrent = Thelp3;  
 else if(GetScreen() == SWORLD) Tmcurrent = Tworld1;
 glPushMatrix();
 glTranslatef(0, 0, -0.0001); 
 glTranslatef(-0.54, -0.41, 0);
 glScalef(0.97, 0.97, 1);
 PaintObject(MenuList, Tmcurrent, GL_QUADS, ISGREY, true, 1.0);
 glPopMatrix(); 
 if(GetScreen() == STITLE)
 {
  glPushMatrix();
  glTranslatef(GetMenuX(), GetMenuY(), 0);
  glScalef(0.4, 0.4, 1); 
  PaintObject(HUDList, Tlice, GL_QUADS, ISGREY, true, 1.0);
  glPopMatrix();
 }
 else if(GetScreen() == SWORLD)
 {
  glPushMatrix();
  glTranslatef(GetWorldX(), GetWorldY(), 0);
  glScalef(0.29, 0.31, 1); 
  PaintObject(HUDList, Tlpid, GL_QUADS, ISGREY, true, 1.0);
  glPopMatrix();      
 }
}

//Puts together and displays the total HUD.
void PaintHUD()
{
 glPushMatrix();

 glTranslatef(-0.412, -0.414, 0);
 double ChangeX = 0.13725;
 glTranslatef(0, 0, -0.0001);
 PaintObject(HUDList, Thud1, GL_QUADS, ISGREY, true, 1.0);
 glTranslatef(ChangeX, 0, 0); 
 PaintObject(HUDList, Thud2, GL_QUADS, ISGREY, true, 1.0); 
 glTranslatef(ChangeX, 0, 0); 
 PaintObject(HUDList, Thud3, GL_QUADS, ISGREY, true, 1.0); 
 glTranslatef(ChangeX, 0, 0); 
 PaintObject(HUDList, Thud4, GL_QUADS, ISGREY, true, 1.0); 
 glTranslatef(ChangeX, 0, 0); 
 PaintObject(HUDList, Thud5, GL_QUADS, ISGREY, true, 1.0);    
 glTranslatef(ChangeX, 0, 0); 
 PaintObject(HUDList, Thud6, GL_QUADS, ISGREY, true, 1.0);
 glTranslatef(-0.26, 0.06, 0.0001);    
 glScalef(0.5, 0.5, 1);
 PaintObject(HUDList, Tlcurrent, GL_QUADS, ISGREY, true, 1.0);    
 glPopMatrix(); 
}

void PaintCursor(int x, int y)
{

 glPushMatrix();
 glEnable( GL_TEXTURE_2D );     
 glBindTexture( GL_TEXTURE_2D, Tcursor.texID);      
 glEnable( GL_BLEND ); 
 glColor4f(1, 1, 1, 1);
 glBegin(GL_QUADS);
   glTexCoord2d(0,0);
   glVertex3f(x-10, y-1, 1);                
   glTexCoord2d(0,1);   
   glVertex3f(x+22, y-1, 1);     
   glTexCoord2d(1,1);   
   glVertex3f(x+22, y+31, 1);     
   glTexCoord2d(1,0);  
   glVertex3f(x-10, y+31, 1);     
 glEnd();
 glPopMatrix();
}

//Sets me up for picking based on color codes.
void StartPicking()
{
 ColorCode = 1;    
}

//Stops it.
void StopPicking()
{
 ColorCode = 0;    
}

//We set the new active colour in the level.
void SetActiveColour(ColourType newActive)
{
 Active = newActive;
}

void SetSkidmark()
{
 Skidmark = true;    
}

void PaintMapBottom()
{
 glPushMatrix();
 glTranslatef(0.0, 0.1, 0.0);
  PaintObject(WallBoundsList, Tbottom, GL_QUADS, ISGREY, false, 1.0); 
 glPopMatrix();
}

//This function paints walls.
void PaintWall(WallTile theWall, int x, int y, bool Reflection)
{
 //If this is a reflection, don't paint the bottom to the level again, otherwise...that'd look weird.
 glPushMatrix();
 glTranslatef(x, 0, y);
 if(!theWall.Hole)  //If there is a hole, we just don't paint anything.
 {
  if(!Reflection)
  {
   if(theWall.Dirty) PaintObject(WallBottomList, Tdirt, GL_QUADS, ISGREY, false, 1.0); 
   else 
   {
    glEnable( GL_BLEND );
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
    PaintObject(WallBottomList, Tbottom, GL_QUADS, ISGREY, false, 0.6); //Paint the bottom.            
   }
  }
   //Now we paint the appropriate walls with the appropriate colour.
  if(theWall.Blocks[0]) PaintObject(WallNorthList, Twall, GL_QUADS, theWall.Colour, false, 1.0);
  if(theWall.Blocks[1]) PaintObject(WallEastList, Twall, GL_QUADS, theWall.Colour, false, 1.0);
  if(theWall.Blocks[2]) PaintObject(WallSouthList, Twall, GL_QUADS, theWall.Colour, false, 1.0); 
  if(theWall.Blocks[3]) PaintObject(WallWestList, Twall, GL_QUADS, theWall.Colour, false, 1.0);   
 }
 else PaintObject(HoleList, Thole, GL_QUADS, ISGREY, false, 1.0);
 
 glPopMatrix();
}

//This paints the particles once everything else has been drawn.

void PaintParticles(bool Anim)
{
 PaintSystem(Anim); //Tell the particle system to draw itself.
}

//This paints the given tile.  So it takes some info from it and determines what and where
//to draw the object.

PlayTile PaintCharacter(PlayTile theTile, bool Anim)
{
 glPushMatrix();
 float AnimSpeed = 0.2; //This constant just moves the object a very little bit
                         //between frames.

 //If the tile should be moving in the Painter(cuz the X,Ys don't match up)
 //Note that we only animate the tile if we've been told to by the Driver.
 //This is to match up a constant framerate across all computers (hopefully).
 if(Anim)
 {
  if(theTile.paintX < theTile.X) 
  {
   theTile.paintX += AnimSpeed;
   if(theTile.paintX > theTile.X) theTile.paintX = theTile.X;
  }
  else if(theTile.paintX > theTile.X) 
  {
   theTile.paintX -= AnimSpeed;
   if(theTile.paintX < theTile.X) theTile.paintX = theTile.X;
  } 
  if(theTile.paintY < theTile.Y) 
  {
   theTile.paintY += AnimSpeed;
   if(theTile.paintY > theTile.Y) theTile.paintY = theTile.Y;
  }
  else if(theTile.paintY > theTile.Y) 
  {
    theTile.paintY -= AnimSpeed;
   if(theTile.paintY < theTile.Y) theTile.paintY = theTile.Y;
  } 
 }

//Then we centre it and scale it down a little.  This is so that
//the object will draw properly in-between walls.
 glTranslatef(theTile.paintX+0.04, 0.1, theTile.paintY-0.05);  
 glScalef(0.92, 0.92, 0.92);  
 //If my tile is 'alive', consider the direction it's facing.  Rotate
 //the object to face that direction.  To be honest, I could be doing
 //this with ALL of my objects...but only Heroes and Enemies have
 //appreciable point ats.
 if(theTile.Alignment == HERO || theTile.Alignment == ENEMY)
 {
  if(theTile.paintM == EAST)
  {
   glTranslatef(0.5, 0, 0.5);
   glRotatef(90, 0, 1, 0);
   glTranslatef(0.5, 0, 0.5);
  }
  if(theTile.paintM == WEST)
  {
   glTranslatef(0.5, 0, 0.5);
   glRotatef(-90, 0, 1, 0);
   glTranslatef(-1.5, 0, 0.5);
  }
  if(theTile.paintM == NORTH)
  {
   glTranslatef(0.5, 0, 0.5);
   glRotatef(180, 0, 1, 0);
   glTranslatef(-0.5, 0, 1.5);
  }    
 }
 
 //Of course, the PointDirection combines with the Face direction.  The code below
 //is held exclusively by arrows at the moment.  Also, an object will have either
 //one or the other and not both.  This also could change.
 
 if(theTile.PointDirection == PEAST)
 {
  glTranslatef(0.5, 0, 0.5);
  glRotatef(-90, 0, 1, 0); 
  glTranslatef(-1.5, 0, 0.5);
 }
 else if(theTile.PointDirection == PWEST)
 {
  glTranslatef(0.5, 0, 0.5);
  glRotatef(90, 0, 1, 0);  
  glTranslatef(0.5, 0, 0.5);     
 }
 else if(theTile.PointDirection == PSOUTH)
 {
  glTranslatef(0.5, 0, 0.5);
  glRotatef(180, 0, 1, 0);
  glTranslatef(-0.5, 0, 1.5);
 }   
 
 //If the object is an inactive colour, we make it smaller so that
 //when it turns on, it grows in size.  Well, I thought it looked neat anyway.
 if((theTile.Colour == ISRED && Active == ISYELLOW) ||
    (theTile.Colour == ISYELLOW && Active == ISRED))
 {
  glScalef(0.7, 0.7, 0.7);
  glTranslatef(0.2, 0, -0.2);
 }
 
 //Use the String to determine which object to paint.
 if(strcmp("PID", theTile.Name) == 0) PaintPid(theTile.Colour, Anim);
 else if(strcmp("CRYSTAL", theTile.Name) == 0)
 {
  PaintCrystal(theTile.Colour);
  RegisterSystem(theTile.paintX, theTile.paintY, 2);
 }  
 else if(strcmp("CLOSEDEXIT", theTile.Name) == 0) PaintExit(theTile.Colour);
 else if(strcmp("OPENEXIT", theTile.Name) == 0) 
 {
  PaintOpenExit(theTile.Colour); 
  RegisterSystem(theTile.paintX, theTile.paintY, 3);  
 }
 else if(strcmp("ALLY", theTile.Name) == 0) PaintAlly(theTile.Colour);
 else if(strcmp("GOLEM", theTile.Name) == 0) PaintGolem(theTile.Colour);
 else if(strcmp("FURY", theTile.Name) == 0) 
 {
  PaintFury(theTile.Colour);
  RegisterSystem(theTile.paintX, theTile.paintY, 1);
 }
 else if(strcmp("FLAME", theTile.Name) == 0) 
 {
  PaintFlame(theTile.Colour);
  RegisterSystem(theTile.paintX, theTile.paintY, 1);
 }
 else if(strcmp("SWITCH", theTile.Name) == 0) 
 {
   if(theTile.Colour == SETRED) PaintRedSwitch(theTile.Colour);
   else if(theTile.Colour == SETYELLOW) PaintYellowSwitch(theTile.Colour);
 }
 else if(strcmp("TRAP", theTile.Name) == 0) PaintTrap(theTile.Colour);
 else if(strcmp("ARROW", theTile.Name) == 0) PaintArrow(theTile.Colour);
 else if(strcmp("HVBLOCK", theTile.Name) == 0) PaintHVBlock(theTile.Colour);
 else if(strcmp("VBLOCK", theTile.Name) == 0) PaintVBlock(theTile.Colour);
 else if(strcmp("HBLOCK", theTile.Name) == 0) PaintHBlock(theTile.Colour);
 else if(strcmp("DBLOCK", theTile.Name) == 0) PaintDBlock(theTile.Colour);
 glPopMatrix(); 
 return theTile;
}

//This function paints the Info string.
void PaintInfo()
{
 PaintString(472, 516, GLUT_BITMAP_HELVETICA_10, Info);      
}

//This function sets the Info string depending on the object that was picked.
void PaintHelp(int Picked)
{
 if(Picked == 0)
 {
  Info = "";  
  Tlcurrent = Tlnothing;
 }
 else if(Picked <= CPID)
 {
  Info = "The penguins are a noble breed.  They are\nworking together to find the crystals and to \nstop the invading forces of the Furies.\n\n But only Pid has the ability to control the ice, \nthe crystals, and the blocks.";     
  Tlcurrent = Tlpenguin;
 }    
 else if(Picked <= CCRYSTAL)
 {
  Info = "One day, these mysterious gems appeared \nalong with the statues of stone and ice. \n\n Pid must find all of them whenever he can \nbefore he can leave the area!";     
  Tlcurrent = Tlcrystal;  
 }
 else if(Picked <= CEXIT) 
 {
  Info = "These exits will lead penguins out of the area.\n\n  Pid must collect all the crystals before it \ncan be used.";    
  Tlcurrent = Tlexit;
 }
 else if(Picked <= CGOLEM)
 {
  Info = "Always surrounding Pid, the golems of \nKouriku rise out of the ground to greet him. \n\n Silent statues which mark a dead era before \nthe need of crystals.  They will protect Pid \nfrom surrounding furies.";     
  Tlcurrent = Tlgolem;
 }
 else if(Picked <= CFURY)
 {
  Info = "The Furies come from an uknown land.  \nThey are invading Freja and know that Pid, \nwith his powers of Kouriku summoning, is the \nonly one who can stand against them.  They \nwill burn him into ashes given the chance.";     
  Tlcurrent = Tlfury;
 }
 else if(Picked <= CFLAME)
 {
  Info = "The Flame Generals of the Fury army are \nthe most devious of all.  Although all \nlittle Furies and other objects fall prey to \nPid's command of the ice, the Flames will \nmove in unexpected ways.";     
  Tlcurrent = Tlfury;  
 } 
 else if(Picked <= CSWITCH)
 {
  Info = "With the powers of the Kouriku and the Furies \nclashing, areas have been pushed out of \nexistence.  These areas cannot be felt without \nthe reinforcing power of an Energy Switch. \n\n Pid, use your best judgment!";
  Tlcurrent = Tlswitch;
 }
 else if(Picked <= CTRAP)
 {
  Info = "Planted by Flame Generals, these Trapmines \nare dangerous.  If a trapmine is touched by \nanything containing spirit, the trap will \nexplode.  However, Pid may be able to use \nthese to his advantage...";     
  Tlcurrent = Tltrap;
 } 
 else if(Picked <= CARROW)
 {
  Info = "Arrows will redirect anything that comes near \nthem.  Using an arrow can surprise and \nconfuse your enemies.  If you want to go for a \nride, give these a shot!";
  Tlcurrent = Tlarrow;
 }
 else if(Picked <= CBOX)
 {
  Info = "Boxes are constructs of the Penguin race in \norder to help Pid.  They aren't as easily \nmoved as Kouriku and only Pid can move \nthem.  They can be useful, or a hindrance. \n\n Hey...that black box isn't one of ours...";     
  Tlcurrent = Tlbox;
 }
}

void ExitGraphics()
{
  freeList(BoxList);
  freeList(BoxRimList);
  freeList(BackgroundList);
  freeList(MenuList);
  freeList(HUDList);   
  freeList(WallBottomList);   
  freeList(WallNorthList);   
  freeList(WallEastList);   
  freeList(WallSouthList);   
  freeList(GolemEndsList);   
  freeList(PidFaceList);   
  freeList(PidBodyList);   
  freeList(PidFrontList);   
  freeList(PidFeetList);   
  freeList(TrapList);   
  freeList(TrapEdgeList);   
  freeList(SwitchBoxList);   
  freeList(SwitchList);   
  freeList(CrystalList);   
  freeList(ExitList);   
  freeList(ArrowTopList);   
  freeList(ArrowRimList);   
  freeList(WallBoundsList);
  freeList(HoleList);
  freeList(ArrowStemList);   
  gluDeleteQuadric(QuadList);  
}

