#ifndef gf_included_Painter_h
#define gf_included_Painter_h

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "Particles.h"
#include "types.h"
#include "PlayTile.h"
#include "WallTile.h"
#include "Faces.h"
/*
Valid painter labels include:
      
      PID
      CRYSTAL
      CLOSEDEXIT
      OPENEXIT
      ALLY
      GOLEM
      FURY  //same as Pid.
      FLAME //Opposite to Pid.
      SWITCH
      DIRT
      TRAP
      ARROW
      HVBLOCK
      VBLOCK
      HBLOCK
      DBLOCK
*/

//Pulled directly from NeHe tutorials.  I'm hoping to go pure Targa for my
//image files, hence making MUCH better code.
typedef	struct										// Create A Structure
{
	GLubyte	*imageData;								// Image Data (Up To 32 Bits)
	GLuint	bpp;									// Image Color Depth In Bits Per Pixel
	GLuint	width;									// Image Width
	GLuint	height;									// Image Height
	GLuint	texID;									// Texture ID Used To Select A Texture
} TextureImage;										// Structure Name


//LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc


PlayTile PaintCharacter(PlayTile theTile, bool Anim);
void LoadGraphics();
//This function loads all the graphics from the Graphics folder.

void PaintBackground();
//This paints the background.

void SetActiveColour(ColourType newActive);
//This tells the Painter what the level's active colour is.  It needs to
//know this so it can paint coloured objects properly.  At the moment, if
//an object is inactive, it lacks textures and is smaller in it's spot.

void PaintHUD();
////This paints the panel.  It shows the level name, crystals left, and helpful information.

void PaintWinner();
//Paints the winning message.

void PaintLoser();
//Paints the losing message.

void PaintGameMenu();
//Paints the in-game menu.

void StartPicking();
//Calculates picking based on the colour given.  So one colour is returned by checking
//where the user clicked exactly.  I then check this colour against the constants in Picking.h.

void StopPicking();
//This stops Picking.  It lets my Painter know that I'm done and it can go back to
//drawing the objects normally.

void PaintTitle();
//Paints the title screen.

void SetSkidmark();
//This tells the painter that Pid just made a move and should paint a skidmark behind him.

void PaintHelp(int Picked);
//Paints the help screens based on the integer passed.  There are only 3 different screens.

void PaintInfo();
//Paints the information for the selected tile.  If it's told to Paint Info, then it takes
//the last selected tile by picking and writes the information.  The info consists of a little
//icon and a paragraph of text defining the object.

void PaintString(double x, double y, void* font, char* string);
//This will paint a string using glutBitmapCharacter.  It makes newlines for \n and skips over _.

void PaintMapBottom();
void PaintParticles(bool Anim);
void PaintCursor(int x, int y);
void PaintWall(WallTile theWall, int x, int y, bool Reflection);
//Paints the walls based on the information it can glean from the walltile.

void ExitGraphics(); //Cleans up memory used by OpenGL.

GLuint LoadTexture( const char * filename, int pwidth, int pheight );
//This function loads *.RAW files based on provided width/height.  The width and height
//are memorized by me.  If you want to open a *.RAW file in the Graphics Folder, check it's
//width and height as specified in the LoadGraphics functions.

#endif
