#include <stdlib.h>
#include "Particles.h"

int EmptyExit = 0;
int EmptySkid = 0;
//These are my three different particle arrays.  Each array is for a different effect.
//That means that two fireballs share the same particle array and so their particles
//will move the same, just in different locations.
Particle Fire[PARTICLELENGTH];
Particle Exit[PARTICLELENGTH];
Particle Lights[PARTICLELENGTH];
Particle SkidMark[PARTICLELENGTH];
GLuint pFire;
float RegisteredSystems[3][50];  //This writes down the location and type of particle
//system that will be used in the current graphics frame.  This is because particles
//need to be drawn after everything else.  So this registered 'list' of systems gets
//checked at the end of each frame.  The painter than calls for each system on the list
//to be used at the game co-ords provided by the list.  If the game registers more than
//50 systems in use, it ignores the remaining ones.  That means the game self-imposes
//a limit of no more than 50 particle systems in-game at any time.

//a type 1 is fire.

void InitializeSystems(GLuint PFire)
{
 pFire = PFire;
 for(int i = 0; i < 50; i++)
 {
  RegisteredSystems[0][i] = 0;       
  RegisteredSystems[1][i] = 0;
  RegisteredSystems[2][i] = 0;  
 }    
}

void RegisterSystem(float x, float y, float type)
{
 int nextPosition = 0;
 while(nextPosition < 50 && RegisteredSystems[2][nextPosition] > 0) nextPosition++;
 
 RegisteredSystems[0][nextPosition] = x;
 RegisteredSystems[1][nextPosition] = y;
 RegisteredSystems[2][nextPosition] = type; 
}

void PaintSystem(bool Update)
{
 int nextPosition = 0;
 bool UpdateFire = Update;
 bool UpdateLights = Update;
 bool UpdateFinish = Update; 
 
 while(nextPosition < 50 && RegisteredSystems[2][nextPosition] > 0)
 {
  glPushMatrix();
  if(RegisteredSystems[2][nextPosition] <= 1)
  {
   glTranslatef(RegisteredSystems[0][nextPosition]+0.5, 0.6, RegisteredSystems[1][nextPosition]-0.5); 
   PaintFire(UpdateFire);
   UpdateFire = false;   
  }
  else if(RegisteredSystems[2][nextPosition] <= 2) 
  {
   glTranslatef(RegisteredSystems[0][nextPosition]+0.4, 0.5, RegisteredSystems[1][nextPosition]-0.5);        
   PaintLights(UpdateLights);
   UpdateLights = false;
  } 
  else if(RegisteredSystems[2][nextPosition] <= 3) 
  {
   glTranslatef(RegisteredSystems[0][nextPosition]+0.4, 0.5, RegisteredSystems[1][nextPosition]-0.5);        
   PaintFinish(UpdateFinish);
   UpdateFinish = false;
  }   
  glPopMatrix();
  RegisteredSystems[2][nextPosition] = 0;  
  nextPosition++;  

 }
}


Particle InitializeSkidParticle()
{
  Particle newParticle;
  newParticle.LifeLeft = 1.0;
  newParticle.Age = (rand()%150)/500.0+0.05;   
  newParticle.R = (rand()%100)/200.0;
  newParticle.G = newParticle.R;
  newParticle.B = newParticle.G;
  
  newParticle.rX = 0.35;
  if(rand()%2) newParticle.rX += 0.3;
  newParticle.rY = 0.0 + (rand()%100)/10000.0;
  newParticle.rZ = -0.8 + (rand()%100)/500.0;
  
  newParticle.vX = -0.5 + (rand()%100)/100.0;
  newParticle.vY = 0.0;
  newParticle.vZ = -(rand()%1000) - 1000;
  
  newParticle.aX = -80 + rand()%160;
  newParticle.aY = 80.0;
  newParticle.aZ = 120.0;
  
  return newParticle;
}

//This sets up swirly particles.  The particle starts somewhere at random and gets pulled
//back towards the centre by the accelerations.
Particle InitializeLightParticle()
{
  Particle newParticle;
  newParticle.LifeLeft = 1.0;
  newParticle.Age = (rand()%100)/500.0+0.04;          
  newParticle.R = 0.0;
  newParticle.G = (rand()%100)/200.0;
  newParticle.B = 1.0 - (rand()%100)/200.0;
  
  newParticle.rX = (rand()%100)/100;
  newParticle.rY = (rand()%100)/100;
  newParticle.rZ = (rand()%100)/100;
  
  newParticle.vX = (rand()%50)*10.0;
  if(newParticle.rX > 0.5) newParticle.vX = -newParticle.vX;
  newParticle.vY = (rand()%50)*10.0;
  if(newParticle.rY > 0.5) newParticle.vY = -newParticle.vY;  
  newParticle.vZ = (rand()%50)*10.0;  
  if(newParticle.rZ > 0.5) newParticle.vZ = -newParticle.vZ;  

  if(newParticle.vX > 0) 
    newParticle.aX = -20.0;
  else newParticle.aX = 20.0;
  if(newParticle.vY > 0) 
    newParticle.aY = -20.0;
  else newParticle.aY = 20.0;
  if(newParticle.vZ > 0) 
    newParticle.aZ = -20.0;
  else newParticle.aZ = 20.0;
  
  return newParticle;
}

//These particles explode outwards with gravity acting on them to pull them back down.
Particle InitializeExitParticle()
{
  Particle newParticle;
  newParticle.LifeLeft = 1.0;
  newParticle.Age = (rand()%100)/1600.0+0.06;          
  newParticle.R = 1.0;
  newParticle.G = 1.0;
  newParticle.B = (rand()%100)/200.0;
  
  newParticle.rX = 0.0;
  newParticle.rY = 0.0;
  newParticle.rZ = 0.0;  
  
  newParticle.vX = ((rand()%200)-100.0)*10.0;
  newParticle.vY = 3000.0 + rand()%1000;
  newParticle.vZ = ((rand()%200)-100.0)*10.0;    

  newParticle.aX = 0.0;
  newParticle.aY = -400.0 - rand()%100;
  newParticle.aZ = 0.0;     
  
  return newParticle;
}

//These particles rise up following a paraboloid, mimicking flames in the crappiest way possible.
Particle InitializeFireParticle()
{
  Particle newParticle;
  newParticle.LifeLeft = 1.0;
  newParticle.Age = (rand()%100)/800.0+0.06;          
  newParticle.R = 1.0 - (rand()%100)/200.0;
  newParticle.G = (rand()%100)/200.0;
  newParticle.B = 0.0;
  
  newParticle.rX = 0.0;
  newParticle.rY = 0.0;
  
  newParticle.vX = ((rand()%50)-26.0)*10.0;
  newParticle.vY = ((rand()%50)-26.0)*10.0;

  newParticle.aX = 0.0;
  newParticle.aY = 50.0;
  
  return newParticle;
}

void InitializeSkidMark()
{
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  SkidMark[i] = InitializeSkidParticle();
 }     
}

void InitializeExit()
{
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  Exit[i] = InitializeExitParticle();
 }     
}

void InitializeLights()
{
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  Lights[i] = InitializeLightParticle();
 }     
}

void InitializeFire()
{
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  Fire[i] = InitializeFireParticle();
 }     
}

//I have different paint functions for each array for a good reason.
//Their lifetimes are redefined sometimes based on the lifetimes of neighbouring
//particles.  So I had to change them.

bool PaintSkidMark(bool Update)
{ 
// glEnable( GL_BLEND );
 glDisable( GL_TEXTURE_2D ); 
// glBlendFunc(GL_SRC_ALPHA, GL_ONE);
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  glColor4f(SkidMark[i].R, SkidMark[i].G, SkidMark[i].B, SkidMark[i].LifeLeft+0.2);
  glBegin(GL_QUADS);
   glTexCoord2d(1,1);
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY, SkidMark[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(SkidMark[i].rX+0.03, SkidMark[i].rY, SkidMark[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(SkidMark[i].rX+0.03, SkidMark[i].rY+0.03, SkidMark[i].rZ+0.03);
   glTexCoord2d(1,1);  
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY+0.03, SkidMark[i].rZ+0.03);         

   glTexCoord2d(1,1);
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY, SkidMark[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(SkidMark[i].rX+0.03, SkidMark[i].rY, SkidMark[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(SkidMark[i].rX+0.03, SkidMark[i].rY+0.03, SkidMark[i].rZ-0.03);
   glTexCoord2d(1,1);  
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY+0.03, SkidMark[i].rZ-0.03);            

   glTexCoord2d(1,1);
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY, SkidMark[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY+0.03, SkidMark[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY+0.03, SkidMark[i].rZ+0.03);
   glTexCoord2d(1,1);  
   glVertex3f(SkidMark[i].rX, SkidMark[i].rY, SkidMark[i].rZ+0.03);      
  glEnd();
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);   
  if(Update)
  {
    SkidMark[i].rX += SkidMark[i].vX/20000;    
    SkidMark[i].rY += SkidMark[i].vY/20000;
    SkidMark[i].rZ += SkidMark[i].vZ/20000;    
  
    SkidMark[i].vX += SkidMark[i].aX/8;
    SkidMark[i].vY += SkidMark[i].aY/20;
    SkidMark[i].vZ += SkidMark[i].aZ/8; 
  
    SkidMark[i].LifeLeft -= SkidMark[i].Age/4;
    if(SkidMark[i].LifeLeft < 0.0)
    {
     EmptySkid++;
     if(EmptySkid > 10*PARTICLELENGTH)
     {
       InitializeSkidMark();
       EmptySkid = 0;
       return false;
     }
    } 
  } 
 }
 return true;
}

void PaintFire(bool Update)
{ 
 glEnable( GL_TEXTURE_2D );     
 glBindTexture( GL_TEXTURE_2D, pFire);      
 glEnable( GL_BLEND );
 glBlendFunc(GL_SRC_ALPHA, GL_ONE);
 float modelview[16];
 glPushMatrix();

 glGetFloatv(GL_MODELVIEW_MATRIX , modelview);

 for(int i=0; i<3; i++ ) 
	for(int j=0; j<3; j++ ) 
    {
		if ( i==j )
			modelview[i*4+j] = 1.0;
		else
			modelview[i*4+j] = 0.0;
	}

 glLoadMatrixf(modelview);
 
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  glColor4f(Fire[i].R, Fire[i].G, Fire[i].B, Fire[i].LifeLeft);
  glBegin(GL_QUADS);
   glTexCoord2d(0,0);
   glVertex3f(Fire[i].rX-0.3, Fire[i].rY-0.3, i/1000.0);                
   glTexCoord2d(0,1);   
   glVertex3f(Fire[i].rX+0.3, Fire[i].rY-0.3, i/1000.0);
   glTexCoord2d(1,1);   
   glVertex3f(Fire[i].rX+0.3, Fire[i].rY+0.3, i/1000.0);
   glTexCoord2d(1,0);  
   glVertex3f(Fire[i].rX-0.3, Fire[i].rY+0.3, i/1000.0);            
  glEnd();
  if(Update)
  {
    Fire[i].rX += Fire[i].vX/100000;    
    Fire[i].rY += Fire[i].vY/100000;   
  
    Fire[i].vX += Fire[i].aX/40;
    Fire[i].vY += Fire[i].aY/40;
  
    Fire[i].LifeLeft -= Fire[i].Age/10;
    if(Fire[i].LifeLeft < 0.0)
    {
     Fire[i] = InitializeFireParticle();           
    } 
  } 
 }
 glPopMatrix();
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
}

void PaintFinish(bool Update)
{
 glDisable( GL_TEXTURE_2D );
 glEnable( GL_BLEND );
 glBlendFunc(GL_SRC_ALPHA, GL_ONE);     
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  glColor4f(Exit[i].R, Exit[i].G, Exit[i].B, Exit[i].LifeLeft+0.2);
  glBegin(GL_QUADS);
   glTexCoord2d(1,1);
   glVertex3f(Exit[i].rX, Exit[i].rY, Exit[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(Exit[i].rX+0.05, Exit[i].rY, Exit[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(Exit[i].rX+0.05, Exit[i].rY+0.05, Exit[i].rZ+0.05);
   glTexCoord2d(1,1);  
   glVertex3f(Exit[i].rX, Exit[i].rY+0.05, Exit[i].rZ+0.05);         

   glTexCoord2d(1,1);
   glVertex3f(Exit[i].rX, Exit[i].rY, Exit[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(Exit[i].rX+0.05, Exit[i].rY, Exit[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(Exit[i].rX+0.05, Exit[i].rY+0.05, Exit[i].rZ-0.05);
   glTexCoord2d(1,1);  
   glVertex3f(Exit[i].rX, Exit[i].rY+0.05, Exit[i].rZ-0.05);            

   glTexCoord2d(1,1);
   glVertex3f(Exit[i].rX, Exit[i].rY, Exit[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(Exit[i].rX, Exit[i].rY+0.05, Exit[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(Exit[i].rX, Exit[i].rY+0.05, Exit[i].rZ+0.05);
   glTexCoord2d(1,1);  
   glVertex3f(Exit[i].rX, Exit[i].rY, Exit[i].rZ+0.05);      
  glEnd();
  
  if(Update)
  {
   Exit[i].rX += Exit[i].vX/10000;    
   Exit[i].rY += Exit[i].vY/10000;
   Exit[i].rZ += Exit[i].vZ/10000;    
  
   Exit[i].vX += Exit[i].aX/4;
   Exit[i].vY += Exit[i].aY/4;
   Exit[i].vZ += Exit[i].aZ/4; 
  
   Exit[i].LifeLeft -= Exit[i].Age/4;
   if(Exit[i].LifeLeft < 0.0)
   {
    EmptyExit++; 
    if(EmptyExit > PARTICLELENGTH - 2)
    {
      InitializeExit();         
      EmptyExit = 0;
    }
   } 
  }
 }
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);  
}

void PaintLights(bool Update)
{
 glEnable( GL_BLEND );
 glBlendFunc(GL_SRC_ALPHA, GL_ONE);     
 glDisable( GL_TEXTURE_2D );     
 for(int i = 0; i < PARTICLELENGTH; i++)
 {
  glColor4f(Lights[i].R, Lights[i].G, Lights[i].B, Lights[i].LifeLeft);
  glBegin(GL_QUADS);
   glTexCoord2d(1,1);
   glVertex3f(Lights[i].rX, Lights[i].rY, Lights[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(Lights[i].rX+0.1, Lights[i].rY, Lights[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(Lights[i].rX+0.1, Lights[i].rY+0.1, Lights[i].rZ+0.1);
   glTexCoord2d(1,1);  
   glVertex3f(Lights[i].rX, Lights[i].rY+0.1, Lights[i].rZ+0.1);         

   glTexCoord2d(1,1);
   glVertex3f(Lights[i].rX, Lights[i].rY, Lights[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(Lights[i].rX+0.1, Lights[i].rY, Lights[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(Lights[i].rX+0.1, Lights[i].rY+0.1, Lights[i].rZ-0.1);
   glTexCoord2d(1,1);  
   glVertex3f(Lights[i].rX, Lights[i].rY+0.1, Lights[i].rZ-0.1);            

   glTexCoord2d(1,1);
   glVertex3f(Lights[i].rX, Lights[i].rY, Lights[i].rZ);                
   glTexCoord2d(0,1);   
   glVertex3f(Lights[i].rX, Lights[i].rY+0.1, Lights[i].rZ);
   glTexCoord2d(1,0);   
   glVertex3f(Lights[i].rX, Lights[i].rY+0.1, Lights[i].rZ+0.1);
   glTexCoord2d(1,1);  
   glVertex3f(Lights[i].rX, Lights[i].rY, Lights[i].rZ+0.1);      
  glEnd();
  
  if(Update)
  {
   Lights[i].rX += Lights[i].vX/10000;    
   Lights[i].rY += Lights[i].vY/10000;
   Lights[i].rZ += Lights[i].vZ/10000;    
  
   Lights[i].vX += Lights[i].aX;
   Lights[i].vY += Lights[i].aY;
   Lights[i].vZ += Lights[i].aZ; 
  
   Lights[i].LifeLeft -= Lights[i].Age;
   if(Lights[i].LifeLeft < 0.0)
   {
    Lights[i] = InitializeLightParticle();
   } 
  }
 }
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);   
}
