#include <windows.h>
#include <mmsystem.h>
#include <stdlib.h>
#include <math.h>

#include "Map.h"
#include "Menu.h"
#include "Painter.h"

KMap theMap; //The map currently being played.

bool GameMenu = false;  //In-game menu is on or off.
bool MoveCamera = false;  //If the user is currently moving the camera with the mouse or not.
bool JustStartedMove = false;

double ResetX = 2;
double TrackX = ResetX;
double ResetY = 7;
double TrackY = ResetY;  //All of these variables change my lookat for the User Mode camera.
double ResetZ = 5;
double TrackZ = ResetZ;
double LookAtX = 2;
double LookAtY = 1;
double LookAtZ = 3;
double Radius;  //The radius (ie distance) to the centre of the map from the camera.
long int myTimer;
bool TitleMenu = true; //In the menu or not.

int OldX = 0;
int OldY = 0;   //The last place the user clicked his right mouse button.
int MouseX = 0;  //The current position of the mouse cursor.
int MouseY = 0;
int Select = 0; //If picking is on or off.
bool UpdateGame = false;
bool ExitGame = false;

void CleanUpExit() //This function deallocates memory and gives the user back his computer.
{
 glutLeaveGameMode();
 ExitGraphics();
 freeTList(theMap.PlayList);
 exit(0);
}

//This function checks for picking based on color codes.
//I got this function directly from the OpenGL lighthouse tutorials and
//messed with it a little.
void CheckPicking()
{
 GLint viewport[4];
 GLubyte pixel[3];

 glGetIntegerv(GL_VIEWPORT,viewport);

 glReadPixels(OldX,viewport[3]-OldY,1,1, GL_RGB,GL_UNSIGNED_BYTE,(void *)pixel);
// printf("%d\n", pixel[0]);
 PaintHelp(pixel[0]); 
}

//Calculates the initial radius to the centre of the map.
double CalculateRadius()
{
 double ChangeX = TrackX - LookAtX;
 double ChangeY = TrackY - LookAtY;
 double ChangeZ = TrackZ - LookAtZ;
 
 double newRadius = sqrt(ChangeX*ChangeX+ChangeY*ChangeY+ChangeZ*ChangeZ);
 return newRadius;
}

//This function ensures that the radius to the centre of the level never
//changes depending on camera motion.
void RecalculateRadius()
{
 double newRadius = CalculateRadius();
 bool Endit = false;
 while(newRadius > Radius && !Endit)
 {
  TrackZ -= 0.02;
  if(TrackZ < -Radius)
  {
   TrackZ = -Radius;
   Endit = true;
  }
  newRadius = CalculateRadius();
 }
 while(newRadius < Radius && !Endit)
 {
  TrackZ += 0.02;
  if(TrackZ > Radius) 
  {
   TrackZ = Radius;
   Endit = true;
  }
  newRadius = CalculateRadius();
 } 
}

void PassiveMouse(int x, int y)
{
 MouseX = x;
 MouseY = y;    
}

//If right mouse button pressed, move the camera.
void MousePressed(int button, int state, int x, int y)
{   
  OldX = x;
  MouseX = OldX;
  OldY = y;
  MouseY = OldY;
  if(button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) MoveCamera = true;
  else if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) Select = 1;
  else if(button == GLUT_RIGHT_BUTTON && state == GLUT_UP) MoveCamera = false;
}

//While the button is held down.
//There are two different camera motions:
        //A semi-trackball mode for User mode.
        //A full freeform camera mode.
void MouseMoved(int x, int y)
{  
 if(!TitleMenu && !GameMenu && MoveCamera)
 {
   TrackY += (y - OldY)/30.0;
   if(TrackY > 7) TrackY = 7;
   if( TrackY < 1) TrackY = 1;
   OldY = y;
   MouseY = OldY;
   TrackX -= (x - OldX)/30.0;
   if(TrackX < -3+(TrackY/2)) TrackX = -3+(TrackY/2);
   if(TrackX > 7-(TrackY/2)) TrackX = 7-(TrackY/2);
   OldX = x;
   MouseX = OldX;
   RecalculateRadius();     
 }
 else
 {
  MouseX = x;
  MouseY = y;    
 }
}

//If the user presses a key...
void KeyboardDown(unsigned char c, int x, int y) 
{
 if(GameMenu)  //If we're currently in-game and looking at the menu...
 {   //If the user just won, the level will quit based on any key.
  if(!Winner(theMap))
  {
   switch (c)
   {      
    if(!Loser(theMap))  //If he's a loser, he can't get out of the menu.
    {       
     case 27:
     case 13:
     case ' ':
       GameMenu = false;
       break;            
    }
    case 'r':   //Give him the option to retry or quit.
    case 'R':
    TitleMenu = false;
    GameMenu = false;
    TrackX = ResetX;
    TrackY = ResetY;
    TrackZ = ResetZ;
    freeTList(theMap.PlayList);    
    theMap = LoadMap(GetLevelByFile());
    break;
    case 'x':
    case 'X':
    OutOfGame();  //If he quits, we're Out-Of-Game.
    TitleMenu = true;   //Back in the title.
    GameMenu = false;  //And no longer in the in-game menu.
    break;   
   }
  }
  else 
  {
    OutOfGame();
    TitleMenu = true;
    GameMenu = false;
  }
 }
 else if(TitleMenu)  //If he's at the title screen.
 {
  switch (c)
  {   //Use Menu.h to scroll through and select options.
    case 27:
      ExitGame = DeProcessOption();
      break;
    case 13:
    case ' ':
      ProcessOption();
      break;
    case 'd':
    case 's':
      IncreaseMenuOption();
      break;
    case 'a': 
    case 'w':
      DecreaseMenuOption();
      break;          
  }          
  if(InGame())   //If we just went into a level...
  {
    TitleMenu = false;
    TrackX = ResetX;
    TrackY = ResetY;
    TrackZ = ResetZ;
    freeTList(theMap.PlayList);    
    theMap = LoadMap(GetLevelByFile());
  }
 }
 else
 {
  bool NotMoving = false;
  if(!theMap.aLevel.Motion) NotMoving = true;
  switch (c) 
  {
    case 27:  //If you press escape, you bring up the in-game menu.
      GameMenu = true;
      break;         
    case 'a':   //these move the stuff on the board around.
         theMap = MoveWest(theMap);
         break;
    case 'd':
         theMap = MoveEast(theMap);
         break;         
    case 's':
         theMap = MoveSouth(theMap);
         break;         
    case 'w':
         theMap = MoveNorth(theMap);
         break;         
    case ' ':
         TrackX = ResetX;
         TrackY = ResetY;
         TrackZ = ResetZ;
      break;  
  }   
  if(theMap.aLevel.Motion && NotMoving) JustStartedMove = true;
 }
  glutPostRedisplay();
}

//This paints the map.  First it paints each wall tile and then each play tile.
void PaintMap()
{     
 if(JustStartedMove)
 {
  SetSkidmark();
  JustStartedMove = false;                   
 }

 bool Reflection = false;
 //When we ask it to give us the reflection... We reverse Y so that it paints the whole map upside-down.
 glPushMatrix();
 glScalef(0.5, 0.5, 0.5); //This shrinks the map so that the animation isn't too jerky.
 PaintMapBottom();
 do
 {
  Reflection = !Reflection;

  glScalef(1, -1, 1);  
  for(int i = 0; i < theMap.aLevel.Size; i++)
    for(int j = 0; j < theMap.aLevel.Size; j++)
      PaintWall(theMap.aLevel.Walls[i][j], i, j, Reflection);
  theMap.PlayList = ResetList(theMap.PlayList);
  while(theMap.PlayList.Current != NULL)
  {
   theMap.PlayList.Current->theTile = PaintCharacter(theMap.PlayList.Current->theTile, UpdateGame);    
   theMap.PlayList = nextNode(theMap.PlayList);
  }
 }while(Reflection == true);
 PaintParticles(UpdateGame);
 glPopMatrix(); 
}

//This function actually sets up and calls all the paint functions.
void DisplayCallback()
{     
  // clear the color buffer
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  // set up camera  
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective( 45, 1.3, 1.0, 1200.0 ); 
  glMatrixMode( GL_MODELVIEW );
  double modelview[16];  
  glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
  double modelview2[16];  //This is a backup, in case I need the modelview later.
  glGetDoublev(GL_MODELVIEW_MATRIX, modelview2);    
   
  glLoadIdentity();
  if(!TitleMenu) //So if we're in-game...
  {
    //Actually set up and draw the spotlight.  The spotlight is wherever the camera is.
   GLfloat position[] = {0.0, 0.0, 0.0, 1.0};
   GLfloat position1[] = {0.0, 0.0, 0.0, 0.0};
   GLfloat spot_direction[] = { 0.0,0.0,-1.0};
   glLightfv (GL_LIGHT0, GL_POSITION, position);  
   glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);
   //If we're picking, then we tell Painter to draw the objects with the right colours.
   if(Select) StartPicking();
   //Draw the background relative to the camera.
   PaintBackground(); 
   //Draw the Hud relative to the camera.
   PaintHUD();
   
   if(Winner(theMap)) PaintWinner();  //Draw these screens if the level just ended.
   else if(Loser(theMap)) PaintLoser();
   else if(GameMenu) PaintGameMenu(); //If the menu is on, better paint it.

   //Otherwise we just alter the lookat to give us that trackball feel.
   gluLookAt(TrackX, TrackY, TrackZ, LookAtX, LookAtY, LookAtZ, 0, 1, 0); 
   glLightfv(GL_LIGHT1, GL_POSITION, position1);
   if(Select)  //If we're in picking mode, disable lighting.
   {
    glDisable(GL_LIGHT0);
    glDisable(GL_LIGHT1);
   }
   else 
   {
    glEnable(GL_LIGHT0);  
    glEnable(GL_LIGHT1); 
   }
   PaintMap(); //Paint the actual level.

   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   gluOrtho2D(0, 800, 0, 600);  //Get all 2D and draw the strings which
   glScalef(1, -1, 1); //tell the user information such as level name and help info.
                       //I made the screen 800x600 since I was dealing with a Windowed version
                       //of the game earlier.  It makes no difference that I switched it...since
                       //Ortho...does it...anyway.  Trust me, it's fine.
   glTranslatef(0, -600, 0);
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity(); 
   PaintString(800,800,GLUT_BITMAP_HELVETICA_18," "); //Clearing the colour buffer from painting the rest of the screen.
   char crystals[2];
   if(theMap.PlayList.GetGems)  itoa(theMap.PlayList.NumCollects, crystals, 5);
   else crystals[0] = '0';
   PaintString(350,578,GLUT_BITMAP_HELVETICA_18,crystals);  
   PaintInfo();
   PaintString(130,535,GLUT_BITMAP_HELVETICA_18,GetLevelName());   
   if(!MoveCamera) PaintCursor(MouseX, MouseY); 
   glPopMatrix();
   glPopMatrix();
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);  	
   if(Select) //If we just did a pick, turn it off.
   {
     Select = 0;             
     CheckPicking();     
     StopPicking();
   } 
   else glutSwapBuffers(); //Otherwise, actually show the user what's going on.
  }
  else //Otherwise draw a title screen.
  {
   PaintTitle();       
   if(GetScreen() == SWORLD) //If we're drawing the world map, then display
   {                         //some 2D info to the user.
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(0, 700, 0, 700);
    glScalef(1, -1, 1);
    glTranslatef(0, -700, 0); //This flips my Y coord.  It just makes it easier on me.
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();       
    PaintString(10, 670, GLUT_BITMAP_HELVETICA_18, GetWorldName());
    PaintString(300, 670, GLUT_BITMAP_HELVETICA_18, "Level: ");
    PaintString(400, 670, GLUT_BITMAP_HELVETICA_18, GetLevelName());    
    glPopMatrix();
    glPopMatrix();
   }   
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);    
   glutSwapBuffers();    
  }
 if(UpdateGame) UpdateGame = false;
}

//When we're not doing anything, update the game board.
void onIdle()
{
 if(ExitGame) CleanUpExit();
 
 long int newTime = timeGetTime();
 if(newTime - myTimer > 10)
 {
  UpdateGame = true;
  myTimer = newTime;            
  if(!GameMenu && !TitleMenu)
  {
   if(theMap.AnimateBusy) //Either the painter is busy...
   {
     theMap = CheckAnimation(theMap);
   }
   else   //Or the engine is busy...
   {
    theMap = Interaction(theMap);
    SetActiveColour(theMap.aLevel.Active);
    if(Winner(theMap) || Loser(theMap)) 
    {
    
     GameMenu = true;            
    }
   }
  }
 }
 glutPostRedisplay();
}

int main(int argc, char*argv[])
{
  // create window and rendering context
  myTimer = timeGetTime();
  glutInit( &argc, argv );
  printf("Gimme a sec while I load some stuff...\n");
  glutInitDisplayMode( GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE );
  glutGameModeString( "800x600:32@85" );   //This makes it fullscreen at the specified res.
  glutEnterGameMode();  
//  glutCreateWindow("Kouriku Object Viewer");
  // register display callback
  glutDisplayFunc( DisplayCallback );
  glutKeyboardFunc( KeyboardDown );
  glutIdleFunc(onIdle); 
  glutMouseFunc( MousePressed );
  glutMotionFunc( MouseMoved ); 
  glutPassiveMotionFunc( PassiveMouse );
  glutSetCursor(GLUT_CURSOR_NONE);
  
 // glViewport( 0, 0, 700, 700 ); 
  glEnable( GL_DEPTH_TEST );
  
  LoadGraphics();  //Load all the textures.  This takes awhile since it's about 10Mb worth.
  Radius = CalculateRadius();  
  gluLookAt(ResetX, ResetY, ResetZ, LookAtX, LookAtY, LookAtZ, 0, 1, 0);  
  GLfloat ambient[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 0, 0, 0, 1.0 };
  GLfloat direction[] = { 0.0,0.0,1.0};
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.0001);
  glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 15.0);
  glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, direction);
//Set up my spotlight and global lighting.
  GLfloat ambient1[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat diffuse1[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat specular1[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position1[] = { 0,0,0,0 };
  glLightfv(GL_LIGHT1, GL_AMBIENT, ambient1);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse1);
  glLightfv(GL_LIGHT1, GL_SPECULAR, specular1);
  glLightfv(GL_LIGHT1, GL_POSITION, position1);

  glEnable( GL_LIGHTING );
  glEnable( GL_LIGHT0 );
  glEnable( GL_LIGHT1 );
  glEnable( GL_COLOR_MATERIAL );
  // pass control over to GLUT
  
  glutMainLoop();
  
  return 0;       // never reached
}
