/* 
 * Header file for some numerical routines i just implemented 
 * for fun ;-).
 *
 * For more numerical robust results do not use the LU factorization. 
 * Instead use the PLU factorization. The MW_inverse routine delivers the
 * inverse of A = PLU. The precision of the inverse is ONLY 6-7 digits. This
 * means that for scientific purposes one should not use these functions
 * but instead use the LAPACK library. I hope to enhance this aspect of the
 * code in future versions.
 *
 * Copyright 2003 Mark Wrobel.
 */

/** 
 * Returns the LU factorization of A.
 * A is modified so that it is upper triangular
 * and holds U. n is the dimension of A i.e A is m x n.
 */
int MW_LUfactorization(int m, int n, double *A, double *L);


/** 
 * Returns the LU factorization of A.
 * A is modified so that it is upper triangular
 * and holds U. n is the dimension of A i.e A is m x n.
 * P is the pivot matrix of dimension m x n.
 */
int MW_PLUfactorization(int m, int n, double *A, double *L, double *P);


/* Display a matrix. This method is really just for 
 * debugging. The matrix A is of size m x n.
 */
void MW_displayMatrix(int m, int n, double *A);

/**
 * Transpose A, the result is stored in B.
 */
void MW_transpose(int m, int n, double *A, double *B);

/**
 * Calculates the inverse of a matrix A = P*L*U, where P*L*U is the 
 * the lu factorization with pivoting of A. The inverse of A is
 * returned in B. A must be square with dimension n x n.
 */
int MW_inverse(int n, double *P, double *L, double *U, double *B);
